/********************************************************************
   TORNADO-E64xx Demo Utility.
   USART/USB/RTC/ IRQ test. rev.1A
   (C) MicroLAB Systems, 2002


   Description:
   ------------
	This demo tests USART, USB and RTC ALARM/WDT interrupt requests.

	USART and USB interrupt requests are continuously simulated via 
	DSP software using continuous toggling of interrupt polarity via 
	the corresponding USART/USB control registers (refer to USART and 
	USB documentation for more details).

	RTC ALARM interrupt request is programmed to toggle each minute (refer
	to DS1284/DS1286 datasheet for more details). RTC WDT is programmed
	to toggle each 0.5sec.

	User can observe DSP response to USART/USB/RTC interrupt request 
	toggls via USART PIO I/O pins,	which are configued as the OUTPUTS. 
	PIO	outputs can be observed via oscilloscope.
	
	DSP NMI interrupt is not used in this test.

		Notes:
		------
			1.	This test does not run for E620x/E670x boards. 
				Use E6X_UUIRQ.*, E6X_SXIRQ.* and PXIRQ.* tests instead.
			2. 	This test does not test SIOX and PIOX interrupt requests.
				Use E64XX_PXIRQ.* and E64X_SXIRQ.* tests instead.

	USART/USB/RTC/SX_IRQ-0 interrupt request selectors and the 
	corresponding USART PIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:			DSP IRQ:	USART PIO output:
		-----------			--------	-----------------
		USART_IRQ			EXT_INT6		PIO-2
		USB_IRQ				EXT_INT7		PIO-3
		RTC_WDT (INTA)		EXT_INT5		PIO-1
		RTC_ALARM (INTB)	EXT_INT4		PIO-0


   TORNADO platform:
   -----------------
	MIRAGE JTAG emulator and TI C6000 CCS v.2 tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
	- on-board configuration switch: NOBOOT
***********************************************************************/

#define	__TE6X_SDRAM128M__
#define	__TE6X_SELECT_E64XX__

#include "c6x_def.h"
#include "te6x_dsp.h"



#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR must toggle USART PIO-4 pin.
// NMI ISR is not used in this test.
void nmi_isr(void)
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x10
					 ); 
}


// EXT_INT4 ISR must toggle USART PIO-0 pin
// This ISR is linked to RTC WDT IRQ via RTC INTA pin. In order to restart WDT,
// any of the WDT registers shall be either written or readen.
interrupt void xint4_isr(void)  
{
	TE6X_set_rtc_rg(TE_RTC_WDT_SECONDS_RG,0);
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x1
					 ); 
}


// EXT_INT5 ISR must toggle USART PIO-1 pin.
// This ISR is linked to RTC ALARM IRQ via RTC INTB pin. In order to restart alarm,
// any of the WDT registers shall be either written or readen.
interrupt void xint5_isr(void)  
{
	TE6X_set_rtc_rg(TE_RTC_ALARM_DAY_OF_WEEK_RG,0x81);
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x2
					 ); 
}


// EXT_INT6 ISR must toggle USART PIO-2 pin
interrupt void xint6_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x4
					 ); 
}


// EXT_INT7 ISR must toggle USART PIO-3 pin.
interrupt void xint7_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x8
					 ); 
}


// USB write-to-read pause, which is required for
// operations as WRITE to SCR register, which initializes
// internal state machine operations and requires some
// time to execute
// E6x DSP is very fast and immediate reads may return 
// incorrect results.
_wait_usb()
{ int i;
   for (i=0;i<60;i++);
}




main()
{ int j;
	
	// disable IRQs
	C6X_GINT_DISABLE;

	// init DSP
	TE6X_init_dsp_emif(0);

	// configure EXT_INT polarity upon the E6x board type
	TE6X_INIT_EXT_INT_POLARITY;

	// configure USART PIO port as outputs and clear outputs
	TE6X_set_usart_rg(TE_USART_PCR_RG,0);
	TE6X_set_usart_rg(TE_USART_PVR_RG,0);

	// reset USART IMR
	TE6X_set_usart_rg(TE_USART_CH_A+TE_USART_IMR0_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_A+TE_USART_IMR1_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_B+TE_USART_IMR0_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_B+TE_USART_IMR1_RG,0xff);
	TE6X_set_usart_rg(TE_USART_IPC_RG, 0x01);	// set IRQ PUSH/PULL, active low
	
	// perform software reset of USB controller
    TE6X_set_usb_rg(TE_USB_SBIE_RG, 0);	
    TE6X_set_usb_rg(TE_USB_SBIE1_RG,0);
    TE6X_set_usb_rg(TE_USB_SCR_RG, 0x8);	
	_wait_usb();


	// configure RTC ALARM and WDT registers

		// enable access to RTC controller
	TE6X_SET_RTC_ACCESS_ENABLE_ON;

		// reset TE bit of COMMAND register and freeze RTC clock registers.
		// Set IPSW bit to the '0' state in order to enable RTC-ALARM IRQ at
		// INTB and RTC-WDT IRQ at the INTA. INTA/INTB polarity are configured via
		// bits IBH/IBL and PU/LVL to generate active low LEVEL interrupt.
	TE6X_set_rtc_rg(TE_RTC_CMND_RG,0x00);

		// enable clock oscillator (EOSC=0)
	TE6X_set_rtc_rg(TE_RTC_CLK_MONTH_RG,0);

		// clear ALARM registers in order to generate RTC ALARM each minute
		// (it is not possible to generate RTC alarm faster than once per minute)
	TE6X_set_rtc_rg(TE_RTC_ALARM_MINUTES_RG,0x81);
	TE6X_set_rtc_rg(TE_RTC_ALARM_HOURS_RG,0x81);
	TE6X_set_rtc_rg(TE_RTC_ALARM_DAY_OF_WEEK_RG,0x81);

		// set RTC WDT registers to generate WDT each 0.5sec
	TE6X_set_rtc_rg(TE_RTC_WDT_SECONDS_FRACTION_RG,0x50);
	TE6X_set_rtc_rg(TE_RTC_WDT_SECONDS_RG,0x00);

		// set TE bit of COMMAND register and unfreeze RTC clock registers.
	TE6X_set_rtc_rg(TE_RTC_CMND_RG,0x80);


	// configure EXT_INT4/5/6/7 IRQ source selectors to use RTC/USART/USB IRQ sources.
	// Disable NMI IRQ source selector.
	TE6X_SET_EXT_INT_SEL_OFF(TE6X_NMI_SEL);			// disable NMI

	TE6X_SET_EXT_INT_SEL_RTC_WDT(TE6X_EXT_INT4_SEL);		// configure INT4 to RTC/WDT
	TE6X_SET_EXT_INT_SEL_RTC_ALARM(TE6X_EXT_INT5_SEL);		// configure INT5 to RTC/ALARM

	TE6X_SET_EXT_INT_SEL_USART(TE6X_EXT_INT6_SEL);	// configure EXT_INT6 to USART IRQ
	TE6X_SET_EXT_INT_SEL_USB(TE6X_EXT_INT7_SEL);	// configure EXT_INT7 to USB IRQ

	C6X_INT_ENABLE(4);  	// enable EXT_INT4..7
	C6X_INT_ENABLE(5);
  	C6X_INT_ENABLE(6);		
	C6X_INT_ENABLE(7);

   	C6X_INT_CLEAR_ALL;
	C6X_GINT_ENABLE;    	// set GIE bit 


	j=0;
 	while(1)
   	{
	  // generate USART IRQ on ever 100-th cycle
	  // USART IRQ is generated using toggling of IRQ polarity bit of IPC register.
	  if((j%30)==0)
	  {
		// toggle USART interrupt line using INT polarity bit
		TE6X_set_usart_rg(TE_USART_IPC_RG, 0x03);	
		TE6X_set_usart_rg(TE_USART_IPC_RG, 0x01);	
	  }


	  // generate USB IRQ on ever 100-th cycle
	  // USB IRQ is generated using toggling of IRQ polarity bit of SCR register.
	  if((j%100)==0)
	  {
		// toggle USB interrupt line using INT polarity bit
		TE6X_set_usb_rg(TE_USB_SCR_RG, 0x80);	
		_wait_usb();
		TE6X_set_usb_rg(TE_USB_SCR_RG, 0x00);	
		_wait_usb();
	  }

	  if(++j >=1000) j=0;
   	}

}


