/********************************************************************
   TORNADO-E6x Demo Utility.
   USART/USB IRQ test. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo tests USART and USB interrupt requests.

	USART and USB interrupt requests are continuously simulated via 
	DSP software using continuous toggling of interrupt polarity via 
	the corresponding USART/USB control registers (refer to USART and 
	USB documentation for more details).

	User can observe DSP response to USART/USB interrupt requests via 
	USART PIO I/O pins,	which are configued as the OUTPUTS. PIO
	outputs can be observed via oscilloscope.
	
	For E6202/E6203 the DSP NMI interrupt is tested also and toggles each
	time when active SX_IRQ-0 SIOX IRQ is generated.

		Note:
		------
			This test does not perform testing of SIOX/PIOX IRQs for E62/E67 
			and does not test SX_IRQ-1..2 and PX_IRQ-0..3 IRQs for E6202/E6203. 
			Use SXIRQ.* and PXIRQ.* tests instead.

	USART/USB/SX_IRQ-0 interrupt request selectors and the 
	corresponding USART PIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:	DSP IRQ:	USART PIO output:
		-----------	--------	-----------------
		USART_IRQ	EXT_INT6		PIO-2
		USB_IRQ		EXT_INT7		PIO-3
		SX_IRQ-0	NMI				PIO-4 (E6202/E6203 only)


   TORNADO platform:
   -----------------
	MIRAGE JTAG emulator and TI C6000 CCS v.2 tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  board configuration and demo parameters:
  ----------------------------------------
        on-board configuration switch SW2:
	  - MAP0/NOBOOT/SBSRAM bootmode
***********************************************************************/


#include "c6x_def.h"
#include "te6x_dsp.h"



#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR must toggle USART PIO-4 pin.
void nmi_isr(void)
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x10
					 ); 
}


// EXT_INT4 ISR must toggle USART PIO-0 pin
// EXT_INT4 ISR is not used in this test.
interrupt void xint4_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x1
					 ); 
}


// EXT_INT5 ISR must toggle USART PIO-1 pin
// EXT_INT5 ISR is not used in this test.
interrupt void xint5_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x2
					 ); 
}


// EXT_INT6 ISR must toggle USART PIO-2 pin
interrupt void xint6_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x4
					 ); 
}


// EXT_INT7 ISR must toggle USART PIO-3 pin.
interrupt void xint7_isr(void)  
{
	TE6X_set_usart_rg(TE_USART_PVR_RG,
			   		  TE6X_get_usart_rg(TE_USART_PVR_RG) ^ 0x8
					 ); 
}


// USB write-to-read pause, which is required for
// operations as WRITE to SCR register, which initializes
// internal state machine operations and requires some
// time to execute
// E6x DSP is very fast and immediate reads may return 
// incorrect results.
_wait_usb()
{ int i;
   for (i=0;i<30;i++);
}




main()
{ int j;
	

	// init DSP EMIF areas for using SBSRAM in CE-0 
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);

	// configure EXT_INT polarity upon the E6x board type
	TE6X_INIT_EXT_INT_POLARITY;

	// configure USART PIO port as outputs and clear outputs
	TE6X_set_usart_rg(TE_USART_PCR_RG,0);
	TE6X_set_usart_rg(TE_USART_PVR_RG,0);

	// reset USART IMR
	TE6X_set_usart_rg(TE_USART_CH_A+TE_USART_IMR0_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_A+TE_USART_IMR1_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_B+TE_USART_IMR0_RG,0xff);
	TE6X_set_usart_rg(TE_USART_CH_B+TE_USART_IMR1_RG,0xff);
	TE6X_set_usart_rg(TE_USART_IPC_RG, 0x01);	// set IRQ PUSH/PULL, active low
	
	// perform software reset of USB controller
    TE6X_set_usb_rg(TE_USB_SBIE_RG, 0);	
    TE6X_set_usb_rg(TE_USB_SBIE1_RG,0);
    TE6X_set_usb_rg(TE_USB_SCR_RG, 0x8);	
	_wait_usb();


	// configure EXT_INT6/7 IRQ source selectors to use USART/USB IRQ sources.
	// For E6202/E6203 disable EXT_INT4/5 IRQ soyurce selectors and configure
	// NMI IRQ source selector to use SXIRQ0 source.
	if(TE6202_TE6203_BOARD)
	{
		TE6X_SET_EXT_INT_SEL_SXIRQ0(TE6X_NMI_SEL);		// configure NMI to SXIRQ0 for E6202/E6203
		TE6X_SET_EXT_INT_SEL_OFF(TE6X_EXT_INT4_SEL);	// disable EXT_INT4 for E6202/E6203
		TE6X_SET_EXT_INT_SEL_OFF(TE6X_EXT_INT5_SEL);	// disable EXT_INT5 for E6202/E6203
	}

	TE6X_SET_EXT_INT_SEL_USART(TE6X_EXT_INT6_SEL);	// configure EXT_INT6 to USART IRQ
	TE6X_SET_EXT_INT_SEL_USB(TE6X_EXT_INT7_SEL);	// configure EXT_INT7 to USB IRQ

	C6X_INT_DISABLE(4);  	// disable EXT_INT4..5
	C6X_INT_DISABLE(5);	
  	C6X_INT_ENABLE(6);		// enable EXT_INT6..7
	C6X_INT_ENABLE(7);

   	C6X_INT_CLEAR_ALL;
	C6X_GINT_ENABLE;    /* set GIE bit */


	j=0;
 	while(1)
   	{
	  // generate USART IRQ on ever 100-th cycle
	  // USART IRQ is generated using toggling of IRQ polarity bit of IPC register.
	  if((j%30)==0)
	  {
		// toggle USART interrupt line using INT polarity bit
		TE6X_set_usart_rg(TE_USART_IPC_RG, 0x03);	
		TE6X_set_usart_rg(TE_USART_IPC_RG, 0x01);	
	  }


	  // generate USB IRQ on ever 100-th cycle
	  // USB IRQ is generated using toggling of IRQ polarity bit of SCR register.
	  if((j%100)==0)
	  {
		// toggle USB interrupt line using INT polarity bit
		TE6X_set_usb_rg(TE_USB_SCR_RG, 0x80);	
		_wait_usb();
		TE6X_set_usb_rg(TE_USB_SCR_RG, 0x00);	
		_wait_usb();
	  }

	  if(++j >=1000) j=0;
   	}

}


