/********************************************************************************
  TORNADO-E6x (E62/E67/E6202/E6203/E6414/E6415/E6416) DSP Utilities. Rev.4F.
  (C) MicroLAB Systems Ltd, 1998-2010


  Revision history:
  -----------------
	rev.1A	- '1998, initial release for E62/E67
	rev.2A	- '1999, FLASH support added, structurized utilities
	rev.3A 	- '2000, added support for E6202/E6203
	rev.4A 	- '2002, added support for E64xx (E6414/E6415/E6416)
	rev.4B 	- '2003, EMIF settings have been modified
	rev.4C	- '2003, added E64xx DSP clock frequency ID defs and macros,
			  TE6X_init_dsp_emif() function has been modified to properly
			  configure DSP EMIF upon the DSP clock frequency.
	rev.4D	- Feb-2004, added support for E64xx f/w rev.1A-1 and rev.1A-2 boards with 125Mhz/133MHz
			  EMIF clock correspondingly and extra ID registers for DSP clock frequency and SBSRAM/SDRAM length.
			  TE6X_init_dsp_emif() function has been modified to properly
			  configure the DSP EMIF upon the DSP EMIF clock and SDRAM capacity.
			  __TE6X_SDRAM128M__ compiler option is not required anymore with E64xx boards
			  with f/w rev.1A-1 and rev.1A-2.
	rev.4E	- Oct-2006, fixed some errors:
				1. TE64XX_SDRAN_LEN definition has been modified;
				2. TE6X_dsp_clk_freq() macro has been modified;
				3. all extended API functions return values calling _te6x_set_err() function.
	rev.4F	- 2010, updated DSP EMIF registers settings for Micron SDRAM chips 133MHz '-75' speed grade.


  Notes:
  ------
	1. 	This C-header file is include file for TI C6x compile tools. C6X_DEF.H
		header file for general C6x DSP definitions must also be available for
		inclusion if required.
	2.	This header file is externally controlled from user C-code by run-time
		compiler keys definitions in order to select correct DSP controller type
		and to include/exclude extended E6x API functions for DSP EMIF INIT and
		FLASH programming control.
			__TE6X_SELECT_E64XX__ or __C6X_SELECT_C64XX_DSP__
				- if defined in user code prior inclusion of this header file,
				  then E64xx DSP controller definitions are enabled, otherwise
				  E62/E67/E6202/E6203 controller defs are enabled as default.
				  Correspondingly, __C6X_SELECT_C64XX_DSP__ definition is
				  defined for correct processing of C6X_DEF.H general C6xx DSP
				  header file.

			__TE6X_EXCLUDE_INIT_DSP_EMIF__
				- if defined in user code prior inclusion of this header file,
				  then the DSP EMIF INIT function are excluded from the user
				  source code. Default is INCLUDED. Note, that in case user application
				  comprises of several .C linked together modules, then this run-time
				  compiler key must be included in all modules except for the one module in
				  order to exclude multiple function body appearance.

			__TE6X_SDRAM128M__
				- (E64xx f/w rev.1 (REV1) only) if defined in user code prior inclusion
				  of this header file, then 128Mbyte RAM is installed. Otherwise, 32Mbyte
				  is assumed to be installed. Default is 32Mbyte. This compiler key is not
				  required for E64xx f/w rev.1A-1 and rev.1A-2 boards, which provide the SDRAM
				  length ID via the XMEM_LEN_ID register.


			__TE6X_INCLUDE_AMD_FLASH_CONTROL__
				- if defined in user code prior inclusion of this header file,
				  then the AMD FLASH PROGRAMMING CONTROL function are included
				  into the user source code. Default is EXCLUDED.

				  Note, that in case your application comprises of several
				  .C modules, which are linked together, the this run-time compiler
				  key must be included into only one module in order to exclude
				  multiple function body appearance.


********************************************************************************/

// check for this file has been already included
#ifndef __TE6X_DEF__
#define __TE6X_DEF__ 					1


// general defs
#define	ON								1
#define	OFF								0
#define	TE6X_ON							1
#define	TE6X_OFF						0



//===== DSP-type specific run-time compiler keys processing ==================
// (this is required in order to exclude key confusions)

//	Note:!!
//		If __TE6X_SELECT_E64XX__ or __C6X_SELECT_C64XX_DSP__ compiler keys are
//		not defined in user code, then E62xx/E67 controller definitions are
//		assumed to be used.

// initially, undefine common defs
#ifdef	__TE6X_SELECT_E64XX__
#undef	__TE6X_SELECT_E62_E67_E620X__
#undef	__C6X_SELECT_C64XX_DSP__
#define	__C6X_SELECT_C64XX_DSP__		1	// use C64xx defs in C6X_DEF.H file
#endif

#ifdef	__C6X_SELECT_C64XX_DSP__
#undef	__TE6X_SELECT_E62_E67_E620X__
#undef	__TE6X_SELECT_E64XX__
#define	__TE6X_SELECT_E64XX__		1		// use C64xx defs in C6X_DEF.H file
#endif

#ifndef	__TE6X_SELECT_E64XX__
#undef	__TE6X_SELECT_E62_E67_E620X__
#define	__TE6X_SELECT_E62_E67_E620X__	1
#undef	__TE6X_SDRAM128M__
#undef	__C6X_SELECT_C64XX_DSP__
#endif
//============================================================================


// include C6x DSP defs
#include "c6x_def.h"



#ifdef	__TE6X_SELECT_E62_E67_E620X__
//================== E62/E67/E620x Memory/IO Areas ===========================
// DSP memory areas for E62/E67/E620x DSP controllers

	//----------- SBSRAM area @ EMIF CE-0 area
	// These are default defs used for compatibility with E6x Utilities rev.1x/2x.
		// Default setting for DSP MAP0.
#define	TE6X_SBSRAM_SADDR				0x00000000
#define	TE6X_SBSRAM_EADDR				0x003fffff
#define	TE6X_SBSRAM_LEN					0x00400000
			// aliases
#define	TE6X_SBSRAM_MAP0_SADDR			0x00000000
#define	TE6X_SBSRAM_MAP0_EADDR			0x003fffff
#define	TE6X_SBSRAM_MAP0_LEN			0x00400000

		// settings for DSP MAP1
#define	TE6X_SBSRAM_MAP1_SADDR			0x00400000
#define	TE6X_SBSRAM_MAP1_EADDR			0x007fffff
#define	TE6X_SBSRAM_MAP1_LEN			0x00400000


	//----------- FLASH/EPROM area @ EMIF CE-1 area
	// These are default defs used for compatibility with E6x Utilities rev.1x/2x.
		// Default setting for DSP MAP0.
#define	TE6X_FLASH_SADDR				0x01000000	
#define	TE6X_FLASH_EADDR				0x013fffff
#define	TE6X_FLASH_LEN					0x00400000
			// aliases
#define	TE6X_FLASH_MAP0_SADDR			0x01000000	
#define	TE6X_FLASH_MAP0_EADDR			0x013fffff
#define	TE6X_FLASH_MAP0_LEN				0x00400000

		// settings for DSP MAP1
#define	TE6X_FLASH_MAP1_SADDR			0x01400000
#define	TE6X_FLASH_MAP1_EADDR			0x017fffff
#define	TE6X_FLASH_MAP1_LEN				0x00400000


	//----------- SDRAM area @ EMIF CE-2 area
	// This is default SDRAM bank for all E6x boards with 4Mx16 SDRAM configuration.
	// E6202/E6203 boards must enable SDRAM prior using it.
#define	TE6X_SDRAM_SADDR				0x02000000
#define	TE6X_SDRAM_EADDR				0x02ffffff
#define	TE6X_SDRAM_LEN					0x01000000

	//----------- SDRAM-1 area @ EMIF CE-0 area (E6202/E6203 only)
	// This SDRAM bank #1 is optional and is available at E6202/E6203 only, and is
	// shared with on-board SBSRAM bank at EMIF CE-0 area. Address map for DSP MAP0 and
	// DSP MAP1 are different. E6202/E6203 boards must enable SDRAM prior using it.
		// Default setting for DSP MAP0.
#define	TE6X_SDRAM1_SADDR				0x00000000
#define	TE6X_SDRAM1_EADDR				0x00ffffff
#define	TE6X_SDRAM1_LEN					0x01000000
			//alias
#define	TE6X_SDRAM1_MAP0_SADDR			0x00000000
#define	TE6X_SDRAM1_MAP0_EADDR			0x00ffffff
#define	TE6X_SDRAM1_MAP0_LEN			0x01000000

		// DSP MAP1 default setting
#define	TE6X_SDRAM1_MAP1_SADDR			0x00400000
#define	TE6X_SDRAM1_MAP1_EADDR			0x013fffff
#define	TE6X_SDRAM1_MAP1_LEN			0x01000000



	//----------- I/O peripherals @CE-3
	// Note: peripherals marked as '(*)' are available at E6202/E6203 only.
#define TE6X_USART_SADDR				0x03000000	// r/w: SAB82532 USART (ESCC2) via D0..D7
#define TE6X_USB_SADDR					0x03080000	// r/w: Agere USS-820/825 USB via D0..D7

#define	TE6X_WDT_RESET_ADDR				0x03100000	// w-only, data is ignored
#define TE6X_WDT_ENABLE_ADDR			0x03100004	// WDT-ENABLE: r/w, D0;  E6202/E6203 ID (D1)

#define TE6X_DEV_ID_ADDR				0x03200004	// (*)E6202/E6203 DEV_ID: r, D0..D3
#define TE6X_BMODE_ADDR					0x03200008	// (*)E6202/E6203 BMODE RG: r, D0..D3
#define TE6X_SYS_CNF_ADDR				0x0320000c	// (*)E6202/E6203 XMEM CNTR RG: r/w, D0 (SBSRAM/SDRAM selector)
		// 	alias for XMEM_CNF register for compatibility with old rev.3 E6x utilities boards. 
#define TE6X_XMEM_CNTR_ADDR				TE6X_SYS_CNF_ADDR

#define TE6X_EXT_INT4_SEL_ADDR			0x03180008	// (*)EXT_INT4 src sel (E6202/E6203 only): r/w, D0..D3
#define TE6X_EXT_INT5_SEL_ADDR			0x0318000c	// (*)EXT_INT5 src sel (E6202/E6203 only): r/w, D0..D3
#define TE6X_EXT_INT6_SEL_ADDR			0x03180000	// EXT_INT6 src sel: r/w, D0..D1 (E62/E67), D0..D3 (E6202/E6203)
#define TE6X_EXT_INT7_SEL_ADDR			0x03180004	// EXT_INT6 src sel: r/w, D0..D1 (E62/E67), D0..D3 (E6202/E6203)
#define TE6X_NMI_SEL_ADDR				0x03180010	// (*)NMI src sel (E6202/E6203 only): r/w, D0..D3

#define TE6X_PXSX_RESET_ADDR			0x03200000	// PIOX/SIOX RESET: r/w, D0..D1 (E62/E67), D0..D2 (E6202/E6203)
	// alias for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_PXSX_RUN_ADDR				TE6X_PXSX_RESET_ADDR

#define TE6X_PIOX_SADDR					0x033c0000	// PIOX-16: r/w, D0..D15 (16-bit LSW of 32-bit words), A0..A17
#define TE6X_PIOX_EADDR					0x033fffff
#define TE6X_PIOX_LEN					0x00010000	// PIOX-16 length in data words

#define TE6X_SIOX_SADDR					0x03300000	// SIOX rev.C r/w, D0..D7 (8-bit LSB of 32-bit words), A0..A7
#define TE6X_SIOX_EADDR					0x033000ff
#define TE6X_SIOX_LEN					0x00000040	// SIOX rev.C length in data words
//================================================================
#endif


#ifdef	__TE6X_SELECT_E64XX__
//====================== E64xx Memory/IO Areas ===========================
// DSP memory areas for E64xx DSP controllers
//
// Important note:
// ---------------
//		1. Base addresses for E64x external memory and I/O areas are not
//		   compatible with that for E62xx/E67 DSP controllers.
//		2. External I/O control regs relative addresses are also incompatible.
//		3. All USART, USB, RTC, external I/O control regs, PIOX-16 and SIOX rev.C
//		   subareas of E64xx are allocated at x2 DSP address boundaries (UNSIGNED SHORT),
//		   since they are allocated to 16-bit EMIF-B BCE0 area.
//		4. Starting fm E6x API rev.4D, the EMIF-B@CE1 area for FLASH memory access is configured as
//		   the 8-bit ASYNC area for compatibility with future E64xx board revisions. 

	//----------- SBSRAM area (x64) @ EMIF-A CE-0 area
		// different SBSRAM length defs (in x64 words)
		// available options are 0K, 128Kx64 (1MByte), 256Kx64 (2MByte), and 512Kx64 (4MByte)
#define	TE64XX_SBSRAM_LEN_0K			0
#define	TE64XX_SBSRAM_LEN_128K			0x00100000
#define	TE64XX_SBSRAM_LEN_256K			0x00200000
#define	TE64XX_SBSRAM_LEN_512K			0x00400000
#define	TE64XX_SBSRAM_LEN_MAX			TE64XX_SBSRAM_LEN_512K	// max available so far SBSRAM length for E64xx
			// SBSRAM length aliases in bytes
#define	TE64XX_SBSRAM_LEN_0KB			TE64XX_SBSRAM_LEN_0K
#define	TE64XX_SBSRAM_LEN_1MB			TE64XX_SBSRAM_LEN_128K
#define	TE64XX_SBSRAM_LEN_2MB			TE64XX_SBSRAM_LEN_256K
#define	TE64XX_SBSRAM_LEN_4MB			TE64XX_SBSRAM_LEN_512K
			// aliases for compatibility with E6x API
#define	TE6X_SBSRAM_LEN_0K				TE64XX_SBSRAM_LEN_0K
#define	TE6X_SBSRAM_LEN_128K			TE64XX_SBSRAM_LEN_128K
#define	TE6X_SBSRAM_LEN_256K			TE64XX_SBSRAM_LEN_256K
#define	TE6X_SBSRAM_LEN_512K			TE64XX_SBSRAM_LEN_512K
#define	TE6X_SBSRAM_LEN_MAX				TE64XX_SBSRAM_LEN_MAX
			// SBSRAM length aliases in bytes
#define	TE6X_SBSRAM_LEN_0KB				TE64XX_SBSRAM_LEN_0K
#define	TE6X_SBSRAM_LEN_1MB				TE64XX_SBSRAM_LEN_128K
#define	TE6X_SBSRAM_LEN_2MB				TE64XX_SBSRAM_LEN_256K
#define	TE6X_SBSRAM_LEN_4MB				TE64XX_SBSRAM_LEN_512K


    	// SBSRAM area mapping
		// Notes:
		// 		 1. For E64xx f/w rev.1A-1/2 the particular SBSRAM length ID can be read by DSP from XMEM_LEN_ID_RG
		//		    read-only register.
		//		 2. For E64xx f/w rev.1A the SBSRAM length is returned as 512Kx64 by the TE6X_get_sbsram_len() function.
		// 		 2. Care must be taken, since TE64xx_SBSRAM_EADDR macro returns (TE6X_SBSRAM_SADDR-1)
		// 		    value in case SBSRAM is not installed!!!
#define	TE64XX_SBSRAM_SADDR				0x80000000	   		   		 		 			// SBSRAM start address
#define	TE64XX_SBSRAM_EADDR				(TE64XX_SBSRAM_SADDR+TE6X_get_sbsram_len()-1)  	// last SBSRAM address
#define	TE64XX_SBSRAM_EADDR_MAX			(TE64XX_SBSRAM_SADDR+TE64XX_SBSRAM_LEN_MAX-1) 	// maximum SBSRAM last address
#define TE64XX_SBSRAM_LEN				TE6X_get_sbsram_len()
			// aliases for compatibilty with E620x/E670x API
#define	TE6X_SBSRAM_SADDR				TE64XX_SBSRAM_SADDR
#define	TE6X_SBSRAM_EADDR				TE64XX_SBSRAM_EADDR
#define	TE6X_SBSRAM_LEN					TE64XX_SBSRAM_LEN



	//----------- SDRAM area (x64) @ EMIF-A CE-2 area
		// different SDRAM lengths (in x64 words)
		// available options are 0M, 4Mx64 (32MB) and 16Mx64 (128MB)
#define	TE64XX_SDRAM_LEN_0M				0
#define	TE64XX_SDRAM_LEN_4M				0x02000000
#define	TE64XX_SDRAM_LEN_16M			0x08000000
#define	TE64XX_SDRAM_LEN_MAX			TE64XX_SDRAM_LEN_16M    	// max available so far SDRAM length
		// SDRAM length aliases (in bytes)
#define	TE64XX_SDRAM_LEN_0MB			TE64XX_SDRAM_LEN_0M
#define	TE64XX_SDRAM_LEN_32MB			TE64XX_SDRAM_LEN_4M
#define	TE64XX_SDRAM_LEN_128MB			TE64XX_SDRAM_LEN_16M
			// aliases for compatibility with E6x API
#define	TE6X_SDRAM_LEN_0M				TE64XX_SDRAM_LEN_0M
#define	TE6X_SDRAM_LEN_4M				TE64XX_SDRAM_LEN_4M
#define	TE6X_SDRAM_LEN_16M				TE64XX_SDRAM_LEN_16M
#define	TE6X_SDRAM_LEN_MAX				TE64XX_SDRAM_LEN_MAX
			// SDRAM length aliases (in bytes)
#define	TE6X_SDRAM_LEN_0MB				TE64XX_SDRAM_LEN_0MB
#define	TE6X_SDRAM_LEN_32MB				TE64XX_SDRAM_LEN_32MB
#define	TE6X_SDRAM_LEN_128MB			TE64XX_SDRAM_LEN_128MB

		// SDRAM area mapping
		// Notes:
		// 		 1. For E64xx f/w rev.1A-1/2 the particular SDRAM length ID can be read by DSP from XMEM_LEN_ID_RG
		//		    read-only register.
		//		 2. For E64xx f/w rev.1A the SDRAM length is returned as 4Mx64 by the TE6X_get_sdram_len() function.
		// 		 2. Care must be taken, since TE64xx_SDRAM_EADDR macro returns (TE6X_SDRAM_SADDR-1)
		// 		    value in case SDRAM is not installed!!!
#define	TE64XX_SDRAM_SADDR				0xA0000000	   		   		 		 			// SDRAM start address
#define	TE64XX_SDRAM_EADDR				(TE64XX_SDRAM_SADDR+TE6X_get_sdram_len()-1)   	// last SDRAM address
#define	TE64XX_SDRAM_EADDR_MAX			(TE64XX_SDRAM_SADDR+TE6X_SDRAM_LEN_MAX-1) 	// maximum SDRAM last address
#define TE64XX_SDRAM_LEN				TE6X_get_sdram_len()
			// aliases for compatibilty with E620x/E670x API
#define	TE6X_SDRAM_SADDR              	TE64XX_SDRAM_SADDR
#define	TE6X_SDRAM_EADDR              	TE64XX_SDRAM_EADDR
#define TE6X_SDRAM_LEN                	TE64XX_SDRAM_LEN


	//----------- FLASH/EPROM area @ EMIF-B CE-1 area
	// These are default defs used for compatibility with E6x Utilities.
	// Note, that starting fm the E6x API rev.4D, the EMIF-B@CE1 FLASH area is configred as 8-bit area.
#define	TE64XX_FLASH_SADDR				0x64000000
#define	TE64XX_FLASH_EADDR				0x640fffff
#define	TE64XX_FLASH_LEN				0x00100000
		// aliases for compatibilty with E620x/E670x API
#define	TE6X_FLASH_SADDR				TE64XX_FLASH_SADDR
#define	TE6X_FLASH_EADDR				TE64XX_FLASH_EADDR
#define	TE6X_FLASH_LEN					TE64XX_FLASH_LEN


	//----------- I/O peripherals area @ EMIF-B CE-0
	// Note: 1. peripherals marked as '(*)' are available at E6202/E6203 only.
	//       2. peripherals marked as '(#)' are available at E64xx only.
	// 		 3. I/O vars shall be allocated as UNSIGNED SHORT (!!!) instead of
	//		    for possible UNSIGNED for E62/E67/E620x controllers. For
	//		    compatibility access to all regs shall be done as UNSIGNED SHORT.
	//			All DMA accesses to those peripherals shall be done as UNSIGNED SHORT.
#define TE64XX_USART_SADDR				0x60000000	// r/w: SAB82532 USART (ESCC2) via D0..D7
#define TE64XX_USB_SADDR				0x60020000	// r/w: Agere USS-820/825 USB via D0..D7
#define TE64XX_RTC_SADDR				0x60040000	// (#) E64xx only: r/w: DallasSemi DS1284/DS1286 RTC via D0..D7

#define TE64XX_PIOX_SADDR				0x60100000	// PIOX-16: r/w, D0..D15 (16-bit words), A0..A16
#define TE64XX_PIOX_EADDR				0x6011ffff
#define TE64XX_PIOX_LEN					0x00010000	// PIOX-16 length in data words

#define TE64XX_SIOX_SADDR				0x60120000	// SIOX rev.C r/w, D0..D7 (8-bit LSB of 16-bit words), A0..A6
#define TE64XX_SIOX_EADDR				0x6012007f
#define TE64XX_SIOX_LEN					0x00000040	// SIOX rev.C length in data words

#define	TE64XX_WDT_RESET_ADDR			0x6006000e	// w-only, data is ignored
#define TE64XX_WDT_ENABLE_ADDR			0x60060010	// WDT-ENABLE: r/w, D0/D3;  E64x EX_DEV_ID (D1)

#define TE64XX_DEV_ID_ADDR				0x60060014	// (*)E64 DEV_ID: r, D0..D3
#define TE64XX_BMODE_ADDR				0x60060016	// (*)E64 BMODE RG: r, D0..D3
#define TE64XX_SYS_CNF_ADDR				0x60060018	// (*)E64 XMEM/UTOPIA CNTR RG: r/w,
													//		D0 (SBSRAM/SDRAM selector (E6202/E6203), always '0'
													//		D1 (McBSP1/UTOPIA selector (E64xx)
#define TE64XX_DEV_REV_ID_ADDR			0x6006001a	// (#)E64xx DEV_REV: r, D0..D3 (device revision and DSP clock frequency ID)

#define TE64XX_EXT_INT4_SEL_ADDR		0x60060000	// (*)EXT_INT4 src sel (E6202/E6203 only): r/w, D0..D3
#define TE64XX_EXT_INT5_SEL_ADDR		0x60060002	// (*)EXT_INT5 src sel (E6202/E6203 only): r/w, D0..D3
#define TE64XX_EXT_INT6_SEL_ADDR		0x60060004	// EXT_INT6 src sel: r/w, D0..D1 (E62/E67), D0..D3 (E6202/E6203/E64xx)
#define TE64XX_EXT_INT7_SEL_ADDR		0x60060006	// EXT_INT7 src sel: r/w, D0..D1 (E62/E67), D0..D3 (E6202/E6203/E64xx)
#define TE64XX_NMI_SEL_ADDR				0x60060008	// (*)NMI src sel (E6202/E6203/E64xx only): r/w, D0..D3

#define TE64XX_PXSX_RESET_ADDR			0x60060012	// PIOX/SIOX RESET: r/w, D0..D1 (E62/E67), D0..D2 (E6202/E6203)

			// registers added since E64xx f/w rev.1A-1
#define TE64XX_XMEM_LEN_ID_ADDR			0x6006001c	// (#)E64xx f/w rev.1A-1/2 XMEM_LEN_ID: r, D0..D3 (SDRAM/SBSRAM revision ID)
#define TE64XX_DSP_CLK_ID_ADDR			0x6006001e	// (#)E64xx f/w rev.1A-1/2 DSP_CLK_ID: r, D0..D1 (DSP clock frequency ID)

		// aliases for compatibilty with E620x/E670x API
#define TE6X_USART_SADDR				TE64XX_USART_SADDR
#define TE6X_USB_SADDR					TE64XX_USB_SADDR
#define TE6X_RTC_SADDR					TE64XX_RTC_SADDR

#define	TE6X_WDT_RESET_ADDR				TE64XX_WDT_RESET_ADDR
#define TE6X_WDT_ENABLE_ADDR			TE64XX_WDT_ENABLE_ADDR
#define TE6X_DEV_ID_ADDR				TE64XX_DEV_ID_ADDR
#define TE6X_BMODE_ADDR					TE64XX_BMODE_ADDR
#define TE6X_SYS_CNF_ADDR				TE64XX_SYS_CNF_ADDR
#define TE6X_XMEM_CNF_ADDR				TE64XX_SYS_CNF_ADDR
#define TE6X_DEV_REV_ID_ADDR			TE64XX_DEV_REV_ID_ADDR
#define TE6X_XMEM_LEN_ID_ADDR			TE64XX_XMEM_LEN_ID_ADDR
#define TE6X_DSP_CLK_ID_ADDR			TE64XX_DSP_CLK_ID_ADDR
#define TE6X_EXT_INT4_SEL_ADDR			TE64XX_EXT_INT4_SEL_ADDR
#define TE6X_EXT_INT5_SEL_ADDR			TE64XX_EXT_INT5_SEL_ADDR
#define TE6X_EXT_INT6_SEL_ADDR			TE64XX_EXT_INT6_SEL_ADDR
#define TE6X_EXT_INT7_SEL_ADDR			TE64XX_EXT_INT7_SEL_ADDR
#define TE6X_NMI_SEL_ADDR				TE64XX_NMI_SEL_ADDR

#define TE6X_PXSX_RESET_ADDR			TE64XX_PXSX_RESET_ADDR
#define TE6X_PXSX_RUN_ADDR				TE64XX_PXSX_RESET_ADDR

#define TE6X_PIOX_SADDR					TE64XX_PIOX_SADDR
#define TE6X_PIOX_EADDR					TE64XX_PIOX_EADDR
#define TE6X_PIOX_LEN					TE64XX_PIOX_LEN

#define TE6X_SIOX_SADDR					TE64XX_SIOX_SADDR
#define TE6X_SIOX_EADDR					TE64XX_SIOX_EADDR
#define TE6X_SIOX_LEN					TE64XX_SIOX_LEN
//================================================================
#endif




//================= recommended EMIF configurations ==============
// Rev.4B and later:
//		The EMIF setting values have been updated to match
//		high-speed SBSRAM/SDRAM memory installed onto TORNADO-P6x boards.


#ifdef	__TE6X_SELECT_E62_E67_E620X__
// Old E62/E67 EMIF settings, which are provided for compatibility with
// E6x DSP utilities rev.1x/2x only. These setings are not optimal and do not
// provide max performance in case of fast SBSRAM is installed. These settings
// will run on E6202/E6203 with limited functionality and are not recommended for usage.
#define TE6X_EMIF_GCR_DATA				0x3078      // 1/2 CLK SBSRAM
#define TE6X_EMIF_CE0_DATA				0x40		// SBSRAM
#define TE6X_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TE6X_EMIF_CE2_DATA				0x30		// SDRAM
#define TE6X_EMIF_CE3_DATA				0x31b3c623	// PIOX/SIOX and UTIL area (stb=6, setup=3,hold=3)
#define TE6X_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TE6X_EMIF_SDRAM_TIMER_DATA      1253		// REFR period is shortest for E62/E67
#endif


// Dedicated EMIF settings, which shall be set upon the particular E6x board used.
// These settings are set automatically by TE6X_init_dsp_emif() function (see below).

	//-- new recommended EMIF settings for E62 (DSP_clk=200MHz @ 5ns)
#define TE62_EMIF_GCR_DATA				0x3078      // GCR setting for 1/2 CLK SBSRAM
#define TE62_EMIF_GCR_SBSRAM0WS_DATA	0x307C      // GCR setting for 1x CLK SBSRAM
#define TE62_EMIF_CE0_DATA				0x40		// SBSRAM
#define TE62_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TE62_EMIF_CE2_DATA				0x30		// SDRAM
#define TE62_EMIF_CE3_DATA				0x31b3c623	// PIOX/SIOX and UTIL area (stb=6, setup=3,hold=3)
#define TE62_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TE62_EMIF_SDRAM_TIMER_DATA		1500		// REFR period = 64mS/4K_rows/10us

	//-- new recommended EMIF settings for E67 (DSP_clk=167MHz @ 6ns)
#define TE67_EMIF_GCR_DATA				0x3078      // GCR setting for 1/2 CLK SBSRAM
#define TE67_EMIF_GCR_SBSRAM0WS_DATA	0x307C      // GCR setting for 1x CLK SBSRAM
#define TE67_EMIF_CE0_DATA				0x40		// SBSRAM
#define TE67_EMIF_CE1_DATA				0x8638d823	// FLASH area (stb=24, setup=8,hold=3)
#define TE67_EMIF_CE2_DATA				0x30		// SDRAM
#define TE67_EMIF_CE3_DATA				0x2162c522	// PIOX/SIOX and UTIL area (stb=5, setup=2,hold=2)
#define TE67_EMIF_SDRAM_DATA			0x7116000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TE67_EMIF_SDRAM_TIMER_DATA		1253		// REFR period = 64mS/4K_rows/10us

	//-- recommended EMIF settings for E6202 (DSP_clk=250MHz @ 4ns)
	//  note!!! EMIF CE0 and CE2 config regs shall be configured identically
#define TE6202_EMIF_GCR_DATA			0x3078      // default GCR setting
#define TE6202_EMIF_CE0_SBSRAM_DATA		0x40		// enable CE0 as SBSRAM
#define TE6202_EMIF_CE0_SDRAM_DATA		0x30		// enable CE0 as SDRAM
#define TE6202_EMIF_CE1_DATA			0x87b8de23	// FLASH area (stb=30, setup=8,hold=3)
#define TE6202_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TE6202_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TE6202_EMIF_CE3_DATA			0x31f3c723	// PIOX/SIOX and UTIL area (stb=7, setup=3,hold=3)
#define TE6202_EMIF_SDRAM_DATA			0x7228000	// Trc=70ns, Trp=20ns, Trcd=20ns for Micron -8E SDRAM
#define	TE6202_EMIF_SDRAM_TIMER_DATA	1875		// REFR period = 64mS/4K_rows/8us

	//-- recommended EMIF settings for E6203 (DSP_clk=300MHz @ 3.3ns)
	//  note!!! EMIF CE0 and CE2 config regs shall be configured identically
#define TE6203_EMIF_GCR_DATA			0x3078      // default GCR setting
#define TE6203_EMIF_CE0_SBSRAM_DATA		0x40		// enable CE0 as SBSRAM
#define TE6203_EMIF_CE0_SDRAM_DATA		0x30		// enable CE0 as SDRAM
#define TE6203_EMIF_CE1_DATA			0xc93ce423	// FLASH area (stb=36, setup=12,hold=3)
#define TE6203_EMIF_CE2_SBSRAM_DATA		0x40		// enable CE2 as SBSRAM
#define TE6203_EMIF_CE2_SDRAM_DATA		0x30		// enable CE2 as SDRAM
#define TE6203_EMIF_CE3_DATA			0x4274c923	// PIOX/SIOX and UTIL area (stb=9, setup=4,hold=3)
#define TE6203_EMIF_SDRAM_DATA			0x7228000	// Trc=60ns, Trp=15ns, Trcd=15ns for Micron 150MHz SDRAM
#define	TE6203_EMIF_SDRAM_TIMER_DATA	2272		// REFR period = 64mS/4K_rows/6.6us


	//-- recommended EMIF settings for E64xx
	//	 Notes:
	//   -------
	//		1. Starting fm E6x API rev.4D, the EMIF-B@CE1 area for FLASH memory access is configured as
	//		   the 8-bit ASYNC area for compatibility with future E64xx board revisions. FLASH must be accessed
	//		   as UNSIGNED CHAR (BYTE) words. E6x API rev.4C and earlier configured the EMIF-B@CE1 area
	//		   as 16-bit ASYNC and allowed access to 8-bit FLASH as LSB of 16-bit datawords. No modification of
	//		   application code is required in case supplied FLASH programming utilities are used.

		//~~~ common EMIF registers settings for all revisions of E64xx boards
		// 	  CE-0/2 areas are used/valid for EMIF-A, and CE-0/1 areas are used/valid for EMIF-B
#define TE64XX_EMIFA_GCR_DATA			0x0009277c  // default GCR setting
#define TE64XX_EMIFA_CE0_DATA			0xe0		// configure CE0 as 64-bit SBSRAM
#define TE64XX_EMIFA_CE0_SEC_DATA		0x00000002	// CE-0 secondary ext
#define TE64XX_EMIFA_CE2_DATA			0xd0		// configure CE2 as 64-bit SDRAM
#define TE64XX_EMIFA_CE2_SEC_DATA		0x00000002	// CE-2 secondary ext
#define TE64XX_EMIFA_CE1_DATA			0xffffff03	// reserved, leave as default
#define TE64XX_EMIFA_CE3_DATA			0xffffff03	// reserved, leave as default
#define TE64XX_EMIFB_GCR_DATA			0x0001207c  // default GCR setting
#define TE64XX_EMIFB_CE0_DATA_PRESET	0xffffff13	// pre-setup setting of EMIF-B CE-0 space configuration
													// (required to set 16-bit ASYNC space in EMIF-B CE-0 and perform correct control regs decoding for
													//  further configuraion of EMIF-B CE-0 space register upon the f/w revision, DSP speed and silicon rev)
#define TE64XX_EMIFB_CE2_DATA			0xffffff03	// reserved, leave as default
#define TE64XX_EMIFB_CE3_DATA			0xffffff03	// reserved, leave as default

		//~~~ register specific settings upon the f/w revision, DSP speed and SDRAM capacity
			// EMIF settings for E64xx f/w rev.1A with 500MHz TMX-grade DSP, EMIF-A/B_clk=CPU_CLK/4=125MHz@8ns
#define TE64XX_EMIFA_SDRAM16M_DATA_REV1A_500MHZ			0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TE64XX_EMIFA_SDRAM4M_DATA_REV1A_500MHZ			0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 150MHz SDRAM
#define TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_500MHZ		0x00014d29	// SDRAM ext
#define	TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_500MHZ	976			// REFR period = 64mS@8192rows => 7.81us @ 125MHz(8ns)
#define	TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_500MHZ	1952		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(8ns)
#define TE64XX_EMIFB_CE0_DATA_REV1A_500MHZ				0x21124411	// CNTR_RG/USART/USB/PIOX/SIOX area with sil.rev.1.0 and 125MHz EMIF-B clock (16-bit ASYNC, stb=4, setup=2,hold=1,TA=1)
#define TE64XX_EMIFB_CE1_DATA_REV1A_500MHZ				0x23524d01	// FLASH area (8-bit,stb=13, setup=2,hold=1)
				// aliases to match the defs of E6x API v.4C
#define TE64XX_EMIFA_SDRAM128M_DATA_500M                TE64XX_EMIFA_SDRAM16M_DATA_REV1A_500MHZ
#define TE64XX_EMIFA_SDRAM32M_DATA_500M                 TE64XX_EMIFA_SDRAM4M_DATA_REV1A_500MHZ
#define TE64XX_EMIFA_SDRAM_EXT_DATA_500M                TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_500MHZ
#define	TE64XX_EMIFA_SDRAM128M_TIMER_DATA_500M          TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_500MHZ
#define	TE64XX_EMIFA_SDRAM32M_TIMER_DATA_500M           TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_500MHZ
#define TE64XX_EMIFB_GCR_DATA_500M                      TE64XX_EMIFB_GCR_DATA
#define TE64XX_EMIFB_CE0_DATA_500M                      TE64XX_EMIFB_CE0_DATA_REV1A_500MHZ
#define TE64XX_EMIFB_CE1_DATA_500M                      TE64XX_EMIFB_CE1_DATA_REV1A_500MHZ

			// EMIF settings for E64xx f/w rev.1A with 600MHz DSP, EMIF-A/B_clk=CPU_CLK/6=100MHz@10ns
#define TE64XX_EMIFA_SDRAM16M_DATA_REV1A_600MHZ			0x63115000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TE64XX_EMIFA_SDRAM4M_DATA_REV1A_600MHZ			0x57115000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ		0x00014d29	// SDRAM ext
#define	TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ	781			// REFR period = 64mS@8192rows => 7.81us (10ns)
#define	TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ	1562		// REFR period = 64mS@4096rows => 15.62us (10ns)
#define TE64XX_EMIFB_CE0_DATA_REV1A_600MHZ				0x20d24311	// CNTR_RG/USART/USB/PIOX/SIOX area with sil.rev.1.03 and 100MHz EMIF-B clock (16-bit ASYNC, stb=3, setup=2,hold=1, TA=1)
#define TE64XX_EMIFB_CE1_DATA_REV1A_600MHZ				0x22924a01	// FLASH area (8-bit,stb=10, setup=2,hold=1)
				// aliases to match the defs of E6x API v.4C
#define TE64XX_EMIFA_SDRAM128M_DATA_600M                TE64XX_EMIFA_SDRAM16M_DATA_REV1A_600MHZ
#define TE64XX_EMIFA_SDRAM32M_DATA_600M                 TE64XX_EMIFA_SDRAM4M_DATA_REV1A_600MHZ
#define TE64XX_EMIFA_SDRAM_EXT_DATA_600M                TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ
#define	TE64XX_EMIFA_SDRAM128M_TIMER_DATA_600M          TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ
#define	TE64XX_EMIFA_SDRAM32M_TIMER_DATA_600M           TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ
#define TE64XX_EMIFB_GCR_DATA_600M                      TE64XX_EMIFB_GCR_DATA
#define TE64XX_EMIFB_CE0_DATA_600M                      TE64XX_EMIFB_CE0_DATA_REV1A_600MHZ
#define TE64XX_EMIFB_CE1_DATA_600M                      TE64XX_EMIFB_CE1_DATA_REV1A_600MHZ

			// EMIF settings for E64xx f/w rev.1A-1 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=125MHz@8ns
#define TE64XX_EMIFA_SDRAM16M_DATA_REV1A1				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TE64XX_EMIFA_SDRAM4M_DATA_REV1A1				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
#define TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A1				0x00014d29	// SDRAM ext
#define	TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A1			976			// REFR period = 64mS@8192rows => 7.81us @ 125MHz(8ns)
#define	TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1			1952		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(8ns)
#define TE64XX_EMIFB_CE0_DATA_REV1A1					0x21124411	// CNTR_RG/USART/USB/PIOX/SIOX area 125MHz EMIF-B clock (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TE64XX_EMIFB_CE1_DATA_REV1A1					0x23524d01	// FLASH area (8-bit,stb=13, setup=2,hold=1)

			// EMIF settings for E64xx f/w rev.1A-2 with 600/720/1000MHz DSP, EMIF-A/B_clk=external=133MHz@7.5ns
//#define TE64XX_EMIFA_SDRAM16M_DATA_REV1A2				0x63117000	// 16Mx64: Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
//#define TE64XX_EMIFA_SDRAM4M_DATA_REV1A2				0x57117000	// 4Mx64:  Trc=60ns, Trp=15ns, Trcd=15ns for Micron 133MHz SDRAM
//#define TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A2				0x00014d29	// SDRAM ext
#define TE64XX_EMIFA_SDRAM16M_DATA_REV1A2				0x63228000	// 16Mx64 SDRAM: Trc=66ns, Trp=20ns, Trcd=20ns for Micron -75 SDRAM
#define TE64XX_EMIFA_SDRAM4M_DATA_REV1A2				0x57228000	// 4Mx64 SDRAM:  Trc=66ns, Trp=20ns, Trcd=20ns for Micron -75 SDRAM
#define TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A2				0x00014d2b	// SDRAM ext for Micron -75 SDRAM: Tras=44ns, CAS=3clk, Trrd=15ns, Twr=15ns
#define	TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A2			1041		// REFR period = 64mS@8192rows => 7.81us @ 125MHz(7.5ns)
#define	TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2			2082		// REFR period = 64mS@4096rows => 15.62us @ 125MHz(7.5ns)
#define TE64XX_EMIFB_CE0_DATA_REV1A2					0x21224412	// CNTR_RG/USART/USB/PIOX/SIOX area with 133MHz EMIF-B clock (16-bit ASYNC, stb=4, setup=2,hold=1, TA=1)
#define TE64XX_EMIFB_CE1_DATA_REV1A2					0x23d24e01	// FLASH area (8-bit,stb=14, setup=2,hold=1)


//---- DSP clock def upon the DSP type
// This data is not available for direct read-back via E6x registers. These defs are provided
// for user reference.
#define	TE62_DSP_CLK_FREQ				2.0e+8		// TE62 DSP clock frequency (Hz)
#define	TE67_DSP_CLK_FREQ				1.67e+8		// TE67 DSP clock frequency (Hz)
#define	TE6202_DSP_CLK_FREQ				2.5e+8		// TE6202 DSP clock frequency (Hz)
#define	TE6203_DSP_CLK_FREQ				3.0e+8		// TE6203 DSP clock frequency (Hz)
#define	TE64XX_DSP_500MHZ_CLK_FREQ		5.0e+8		// TE64xx board with TMX-grade 500MHz DSP rev.1 clock frequency (Hz)
#define	TE64XX_DSP_600MHZ_CLK_FREQ		6.0e+8		// TE64xx board with TMS-grade 600MHz DSP clock frequency (Hz)
#define	TE64XX_DSP_720MHZ_CLK_FREQ		7.2e+8		// TE64xx board with TMS-grade 720MHz DSP clock frequency (Hz)
#define	TE64XX_DSP_1000MHZ_CLK_FREQ		1.0e+9		// TE64xx board with TMS-grade 1.0GHz DSP clock frequency (Hz)
//================================================================




//=================== peripherals defs ===========================

#define	TE6X_CNTR_DATA_BITMASK			0xf			// read-back data bitmask for FPGA registers

//--- C6x DSP EXT_INT4..7/NMI interrupt source selectors
// C6x DSP external INT selectors are different for different E6x boards:
// 	E62/E67		- only EXT_INT6/EXT_INT7 selectors are available with reduced selector list.
//				  NMI is not supported.
//	E6202/E6203/E64xx	- EXT_INT4..EXT_INT7 and NMI selectors are available.
// Note: not listed settings are reserved for future expansion and E64xx compatibility,
//		 and will result in missing interrupt source in case this setting is programmed.

	// EXT_INT settings for E62/E67 boards. Also valid for E6202/E6203/E64xx.
#define	TE6X_EXT_INT_SEL_XIRQ2			0	//(!)power-on default for C6x DSP EXT_INT6
											// E62/E67:     selects XIRQ2 (common IRQ2 for SIOX and PIOX-16)
											// E6202/E6203: selects SX_IRQ2
#define	TE6X_EXT_INT_SEL_USART			2   //(!)power-on default for C6x DSP EXT_INT7
#define	TE6X_EXT_INT_SEL_USB			3	// USB IRQ selector
#define	TE62_TE67_EXT_INT_SEL_OFF		1	// selector is OFF (no IRQ source is selected)
#define	TE62_TE67_EXT_INT_SEL_MAX_CODE	3	// max selector code value for E62/E67

	// extended EXT_INT/NMI settings for E6202/E6203/E64xx (not compatible with E62/E67)
#define	TE6X_EXT_INT_SEL_OFF			0xf	// selector is OFF (no IRQ source is selected)
#define	TE6X_EXT_INT_SEL_SXIRQ0			4	// selects SIOX IRQ0
#define	TE6X_EXT_INT_SEL_SXIRQ1			5	// selects SIOX IRQ1
#define	TE6X_EXT_INT_SEL_SXIRQ2			0	// selects SIOX IRQ2 (alias for TE6X_EXT_INT_SEL_SX_XIRQ2)
#define	TE6X_EXT_INT_SEL_PXIRQ0			8	// selects PIOX-16 IRQ0
#define	TE6X_EXT_INT_SEL_PXIRQ1			9	// selects PIOX-16 IRQ1
#define	TE6X_EXT_INT_SEL_PXIRQ2			0xa	// selects PIOX-16 IRQ2
#define	TE6X_EXT_INT_SEL_PXIRQ3			0xb	// selects PIOX-16 IRQ3
#define	TE6X_EXT_INT_SEL_WDT			0xc	// selects WDT output interrupt


#ifdef	__TE6X_SELECT_E64XX__
	// E64xx extended EXT_INT/NMI settings (E64xx only)
#define	TE6X_EXT_INT_SEL_RTC_WDT		0xd	// selects RTC WDT interrupt (INTA)
#define	TE6X_EXT_INT_SEL_RTC_ALARM		0xe	// selects RTC ALARM interrupt (INTB)
#endif



//---- PIOX/SIOX Reset Control
	// PIOX reset control.
		// Old defs. Not recommended for usage. Are provided for compatibility
		// with E6x utils rev.2A and earlier.
#define	TE6X_PX_RUN_BITMASK				1
#define	TE6X_PX_RESET					0
#define	TE6X_PX_RUN						1
		// New defs. Recommended for usage.
#define	TE6X_PX_RESET_BITMASK			1
#define	TE6X_PX_RESET_ON				0
#define	TE6X_PX_RESET_OFF				1

	// SIOX-A rev.B site (available at all E6x boards)
		// Old defs. Not recommended for usage. Are provided for compatibility
		// with E6x utils rev.2A and earlier.
#define	TE6X_SX_RUN_BITMASK				2
#define	TE6X_SX_RESET					0
#define	TE6X_SX_RUN						2
			// aliases
#define	TE6X_SXA_RUN_BITMASK			2
#define	TE6X_SXA_RESET					0
#define	TE6X_SXA_RUN					2

		// New defs. Recommended for usage.
#define	TE6X_SX_RESET_BITMASK			2
#define	TE6X_SX_RESET_ON				0
#define	TE6X_SX_RESET_OFF				2
			// aliases
#define	TE6X_SXA_RESET_BITMASK			2
#define	TE6X_SXA_RESET_ON				0
#define	TE6X_SXA_RESET_OFF				2


	// SIOX-B (MXSIOX extender) rev.B site (available at E6202/E6203/E64xx only)
		// Old style defs. Not recommended for usage. Although E62/E67 does not contain MXSIOX
		// site connector, these defs are provided for style compatibility with E6x utils rev.2A and earlier.
#define	TE6X_SXB_RUN_BITMASK			4
#define	TE6X_SXB_RESET					0
#define	TE6X_SXB_RUN					4

		// New defs. Recommended for usage.
#define	TE6X_SXB_RESET_BITMASK			4
#define	TE6X_SXB_RESET_ON				0
#define	TE6X_SXB_RESET_OFF				4

	// bitshift data for universal macros using ON/OFF defs
#define	TE6X_PX_RESET_BITSHIFT			0	
#define	TE6X_SXA_RESET_BITSHIFT			1
#define	TE6X_SXB_RESET_BITSHIFT			2

	// universal defs if used via shift bits and universal macros
#define	TE6X_PXSX_RESET_ON				0	
#define	TE6X_PXSX_RESET_OFF				1


//---- WDT enable register defs.
//		Contain WDT enable bit only (D0), extended dev ID bit (D1) (E6202/E6203/E64) and
//		RTC_WDT enable bit (D3) for E64xx and RTC access enable bit (D2) for E64xx.
//
//		Extended dev ID bit (D1) is used to recognize between old E62/E67 boards and E6202/E6203/E64xx
//		boards, which are using DEV_ID register, which is not available at E62/E67.
//
//		Particular recognition between E62/E67 boards is performed via C6x DSP on-chip CSR
//		register (refer to macros below). Particular recognition between E6202/E6203/E64xx boards is
//		performed via DEV_ID register (refer to macros below).
//
//		RTC_ACCESS enable bit (D2) is availabe at E64x only and is used to enable
//		access to RTC controller. This minimizes risk to corrupt contents of RTC
//		registers and NvRAM on accidental access to RTC due to DSP hangs on or
//		on E64xx controller power ON when no BOOT mode is set (DSP will execute
//		invalid code). RTC_ACCESS enable bit defaults to '0' after power on.
//
//		RTC_WDT enable bit (D3) is availabe at E64x only and is used to enable
//		RTC on-chip WDT instead of conventional WDT feature available at all T-E
//		controllers. RTC on-chip WDT feature programmable duration with high resolution.

#define	TE6X_WDT_ENABLE_BITMASK			0x1
#define	TE6X_WDT_ENABLE_BITSHIFT		0	// bitshift for universal macros (ON/OFF)
#define	TE6X_WDT_ENABLE_OFF				0x0
#define	TE6X_WDT_ENABLE_ON				0x1

#define	TE6X_XDEV_ID_BITMASK			2
#define	TE6X_XDEV_ID_BITSHIFT			1	// bitshift for universal macros
#define	TE6X_XDEV_ID_OFF				0	// E62/E67 boards only
#define	TE6X_XDEV_ID_ON					2	// E6202/E6203/etc, which are decoded via DEV_ID register

#define	TE6X_RTC_WDT_ENABLE_BITMASK		0x8
#define	TE6X_RTC_WDT_ENABLE_BITSHIFT	3	// bitshift for universal macros (ON/OFF)
#define	TE6X_RTC_WDT_ENABLE_OFF			0x0
#define	TE6X_RTC_WDT_ENABLE_ON			0x8

#define	TE6X_RTC_ACCESS_ENABLE_BITMASK	0x4
#define	TE6X_RTC_ACCESS_ENABLE_BITSHIFT	2	// bitshift for universal macros (ON/OFF)
#define	TE6X_RTC_ACCESS_ENABLE_OFF		0x0
#define	TE6X_RTC_ACCESS_ENABLE_ON		0x4

//---- Device ID register defs (E6202/E6203/E64xx only)
// This register is avalable at E6202/TE6203/E64xx boards only and can be used for recogintion
// between E6202/E6203/E64xx boards. DSP software must first read XDEV_ID bit of WDT_EN register
// and ensure that it is set to '1', otherwise these are E62/E67 boards and DEV_ID register does
// not present.
#define	TE6X_DEV_ID_BITMASK				0xf	// E6202 device ID
#define	TE6202_DEV_ID					2	// E6202 device ID
#define	TE6203_DEV_ID					3	// E6203 device ID
#define	TE6414_DEV_ID					8	// E6414 device ID
#define	TE6415_DEV_ID					9	// E6415 device ID
#define	TE6416_DEV_ID					0xa	// E6416 device ID


#ifdef	__TE6X_SELECT_E64XX__
//---- board & f/w REVISION ID register defs (E64xx only)
// This register is avalable at E64xx boards only and can be used for recogintion
// of the board revision ID and f/w revision ID
#define	TE64XX_REV_ID_BITMASK			0xf	// E64xx revision ID code bitmask

#define	TE64XX_REV_ID_REV1				0x1	// f/w rev.1A (initial) for E64xx board rev.1A (500MHz TMX chips and 600MHz TMS chips)
#define	TE64XX_REV_ID_REV1_1			0x2	// f/w rev.1A-1 for E64xx board rev.1A (600MHz, 720MHz, 1GHz chips with 125MHz EMIF-A/B clock)
#define	TE64XX_REV_ID_REV1_2			0x3	// f/w rev.1A-2 for E64xx board rev.1A (600MHz, 720MHz, 1GHz chips with 133MHz EMIF-A/B clock)
		// aliases for easy association with f/w rev.1A and rev.1A-1 boards
#define	TE64XX_REV_ID_REV1A				TE64XX_REV_ID_REV1
#define	TE64XX_REV_ID_REV1A1			TE64XX_REV_ID_REV1_1
#define	TE64XX_REV_ID_REV1A2			TE64XX_REV_ID_REV1_2
			// E6x API aliases
#define	TE6X_REV_ID_BITMASK				TE64XX_REV_ID_BITMASK
#define	TE6X_REV_ID_REV1				TE64XX_REV_ID_REV1
#define	TE6X_REV_ID_REV1_1				TE64XX_REV_ID_REV1_1
#define	TE6X_REV_ID_REV1_2				TE64XX_REV_ID_REV1A2
#define	TE6X_REV_ID_REV1A				TE6X_REV_ID_REV1
#define	TE6X_REV_ID_REV1A1				TE6X_REV_ID_REV1_1
#define	TE6X_REV_ID_REV1A2				TE6X_REV_ID_REV1_2



//---- DSP clock ID register (E64xx only with f/w rev.1A-1 and later)
// This register is avalable at E64xx boards with f/w rev.1A-1/2 and later and can be used for identification
// of the DSP clock frequency.
#define	TE6X_DSP_CLK_FREQ_ID_BITMASK		0x3	// C64xx DSP clock frequency ID code bitmask

#define	TE64XX_DSP_CLK_FREQ_ID_600MHZ		0x0	// C64xx DSP with 600MHz clock
#define	TE64XX_DSP_CLK_FREQ_ID_720MHZ		0x1	// C64xx DSP with 720MHz clock
#define	TE64XX_DSP_CLK_FREQ_ID_1000MHZ		0x2	// C64xx DSP with 1GHz clock
			// E6x API aliases
#define	TE6X_DSP_CLK_FREQ_ID_600MHZ			TE64XX_DSP_CLK_FREQ_ID_600MHZ
#define	TE6X_DSP_CLK_FREQ_ID_720MHZ			TE64XX_DSP_CLK_FREQ_ID_720MHZ
#define	TE6X_DSP_CLK_FREQ_ID_1000MHZ		TE64XX_DSP_CLK_FREQ_ID_1000MHZ

//---- External memory capacity (length) ID register (E64xx only with f/w rev.1A-1 and later)
// This register is avalable at E64xx boards with f/w rev.1A-1 and later and can be used for identification
// of the on-board SBSRAM/SDRAM memories capacity.
#define	TE64XX_SBSRAM_LEN_ID_BITMASK		0x03	// SBSRAM capacity ID bitmask
#define	TE64XX_SDRAM_LEN_ID_BITMASK			0x0c	// SDRAM capacity ID bitmask
			// TE6x aliases
#define	TE6X_SBSRAM_LEN_ID_BITMASK			TE64XX_SBSRAM_LEN_ID_BITMASK
#define	TE6X_SDRAM_LEN_ID_BITMASK			TE64XX_SDRAM_LEN_ID_BITMASK

		// SBSRAM length ID
#define	TE64XX_SBSRAM_LEN_ID_0K				0		// SBSRAM is not installed
#define	TE64XX_SBSRAM_LEN_ID_128K			1		// SBSRAM capacity is 128Kx64
#define	TE64XX_SBSRAM_LEN_ID_256K			2		// SBSRAM capacity is 256Kx64
#define	TE64XX_SBSRAM_LEN_ID_512K			3		// SBSRAM capacity is 512Kx64
			// aliases (for length in bytes)
#define	TE64XX_SBSRAM_LEN_ID_0KB			TE64XX_SBSRAM_LEN_ID_0K
#define	TE64XX_SBSRAM_LEN_ID_512KB			TE64XX_SBSRAM_LEN_ID_128K
#define	TE64XX_SBSRAM_LEN_ID_1MB			TE64XX_SBSRAM_LEN_ID_256K
#define	TE64XX_SBSRAM_LEN_ID_2MB			TE64XX_SBSRAM_LEN_ID_512K
			// E6x API aliases
#define	TE6X_SBSRAM_LEN_ID_0K				TE64XX_SBSRAM_LEN_ID_0K
#define	TE6X_SBSRAM_LEN_ID_128K				TE64XX_SBSRAM_LEN_ID_128K
#define	TE6X_SBSRAM_LEN_ID_256K				TE64XX_SBSRAM_LEN_ID_256K
#define	TE6X_SBSRAM_LEN_ID_512K				TE64XX_SBSRAM_LEN_ID_512K
				// aliases (for length in bytes)
#define	TE6X_SBSRAM_LEN_ID_0KB				TE6X_SBSRAM_LEN_ID_0K
#define	TE6X_SBSRAM_LEN_ID_512KB			TE6X_SBSRAM_LEN_ID_128K
#define	TE6X_SBSRAM_LEN_ID_1MB				TE6X_SBSRAM_LEN_ID_256K
#define	TE6X_SBSRAM_LEN_ID_2MB				TE6X_SBSRAM_LEN_ID_512K


		// SDRAM length ID
#define	TE64XX_SDRAM_LEN_ID_0M				0		// SDRAM is not installed
#define	TE64XX_SDRAM_LEN_ID_4M				4		// SDRAM capacity is 4Mx64
#define	TE64XX_SDRAM_LEN_ID_16M				8		// SDRAM capacity is 16Mx64
			// aliases (for length in bytes)
#define	TE64XX_SDRAM_LEN_ID_0MB				TE64XX_SDRAM_LEN_ID_0M
#define	TE64XX_SDRAM_LEN_ID_32MB			TE64XX_SDRAM_LEN_ID_4M
#define	TE64XX_SDRAM_LEN_ID_128MB			TE64XX_SDRAM_LEN_ID_16M

			// E6x API aliases
#define	TE6X_SDRAM_LEN_ID_0M				TE64XX_SDRAM_LEN_ID_0M
#define	TE6X_SDRAM_LEN_ID_4M				TE64XX_SDRAM_LEN_ID_4M
#define	TE6X_SDRAM_LEN_ID_16M				TE64XX_SDRAM_LEN_ID_16M
				// aliases (for length in bytes)
#define	TE6X_SDRAM_LEN_ID_0MB				TE6X_SDRAM_LEN_ID_0M
#define	TE6X_SDRAM_LEN_ID_32MB				TE6X_SDRAM_LEN_ID_4M
#define	TE6X_SDRAM_LEN_ID_128MB				TE6X_SDRAM_LEN_ID_16M

#endif


//---- SYS_CNF/XMEM register defs (E6202/E6203/E64xx only)
// SYS_CNF register is available at E6202/E6203/E64 boards and is used to define external memory
// type (SBSRAM or SDRAM), which is connected to EMIF CE-0 and CE-2 DSP areas (E6202/E6203 only),
// and to read-back system configuration as McBSP1/UTOPIA I/F selector for E64xx.
// XRAM selection for E6202/E6203 is required since C6202/C6203 DSPs do not support simultaneous
// support of SBSRAM and SDRAM. Note, that user DSP software must also configure the EMIF GCR and
// the corresponding EMIF CE control
// registers to meet the external memory type.
#define	TE6X_XMEM_BITMASK					1
#define	TE6X_XMEM_SBSRAM					0	// in case external memory is SBSRAM
#define	TE6X_XMEM_SDRAM						1	// in case external memory is SDRAM

#define	TE6X_MCBSP1_UTOPIA_IF_SEL_BITMASK	2	// (r) latched on the DSP start
#define	TE6X_MCBSP1_IF_SEL					0	// in case external I/F is McBSP1
#define	TE6X_UTOPIA_IF_SEL					2	// in case external I/F is UTOPIA

//---- BMODE register defs (E6202/E6203/E64xx only)
// BMODE read-only register is available at E6202/E6203/E64x boards and is used to return
// current DSP bootmode and the DSP memory map code. BMODE register is latched on release
// of the DSP reset signal.
//
// E6202/E6203 boards the returned bootmodes are actually the DSP bootmode codes, which
// are supported. Non-specified bootmodes are reserved and will result in incorrect
// device operation if those are set.
//
// E67 boards always boot with MAP0 and do not have BMODE read-back rg.
//
// E64xx boards does not have MAP definition for bootmode, and, for compatibilit,
// the returned bootmod code matches MAP1 list for E6202/E6203.
#define	TE6X_BMODE_BITMASK				0xf

	// supported bootmodes with DSP MAP0
#define	TE6X_BMODE_MAP0_NONE_SBSRAM		0x3	// MAP0, no boot, SBSRAM at CE0/CE2 (configued via EMIF and XMEM_CNF rg)
#define	TE6X_BMODE_MAP0_NONE_SDRAM		0x1	// MAP0, no boot, SDRAM at CE0/CE2 (configued via EMIF and XMEM_CNF rg)
#define	TE6X_BMODE_MAP0_FLASH8_SBSRAM	0xb	// MAP0, boot fm 8-bit FLASH, SBSRAM at CE0/CE2 (configued via EMIF and XMEM_CNF rg)
#define	TE6X_BMODE_MAP0_FLASH8_SDRAM	0x9	// MAP0, boot fm 8-bit FLASH, SDRAM at CE0/CE2 (configued via EMIF and XMEM_CNF rg)
#define	TE6X_BMODE_MAP0_HPI				0x6	// MAP0, boot fm HPI, SBSRAM at CE0/CE2 (configued via XMEM_CNF rg only)

	// supported bootmodes with DSP MAP1
#define	TE6X_BMODE_MAP1_NONE_SBSRAM		0x5	// MAP1, no boot, SBSRAM at CE0/CE2 (configued via XMEM_CNF rg only)
#define	TE6X_BMODE_MAP1_FLASH8_SBSRAM	0xd	// MAP1, boot fm 8-bit FLASH, SBSRAM at CE0/CE2 (configued via XMEM_CNF rg only)
#define	TE6X_BMODE_MAP1_HPI_SBSRAM		0x7	// MAP1, boot fm HPI, SBSRAM at CE0/CE2 (configued via XMEM_CNF rg only)

	// E64xx supported bootmodes
#define	TE64XX_BMODE_NONE				0x5	// no boot
#define	TE64XX_BMODE_FLASH8				0xd	// boot fm 8-bit FLASH
#define	TE64XX_BMODE_HPI				0x7	// boot fm HPI



//---- SAB82532 USART (ESCC2) defs
// Note!!!: no bit defs are provided for USART registers.
//	        User must do it himself upon particular application.

#define TE6X_USART_DATA_BITMASK			0xff	// read-back data bitmask

	//	registers offset definition
#define TE_USART_CH_A					0		// channel-A reg base addr
#define TE_USART_CH_B					0x40	// channel-B reg base addr

#define	TE_USART_RFIFO_RG				0x0		// registers for HDLC mode
#define	TE_USART_TFIFO_RG				0x0
#define	TE_USART_STAR_RG				0x20
#define	TE_USART_CMDR_RG				0x20
#define	TE_USART_RSTA_RG				0x21
#define	TE_USART_PRE_RG					0x21
#define	TE_USART_MODE_RG				0x22
#define	TE_USART_TIMR_RG				0x23
#define	TE_USART_XAD1_RG				0x24
#define	TE_USART_XAD2_RG				0x25
#define	TE_USART_RAH1_RG				0x26
#define	TE_USART_RAH2_RG				0x27
#define	TE_USART_RAL1_RG				0x28
#define	TE_USART_RHCR_RG				0x29
#define	TE_USART_RAL2_RG				0x29
#define	TE_USART_RBCL_RG				0x2A
#define	TE_USART_XBCL_RG				0x2A
#define	TE_USART_RBCH_RG				0x2B
#define	TE_USART_XBCH_RG				0x2B
#define	TE_USART_CCR0_RG				0x2C
#define	TE_USART_CCR1_RG				0x2D
#define	TE_USART_CCR2_RG				0x2E
#define	TE_USART_CCR3_RG				0x2F
#define	TE_USART_TSAX_RG				0x30
#define	TE_USART_TSAR_RG				0x31
#define	TE_USART_XCCR_RG				0x32
#define	TE_USART_RCCR_RG				0x33
#define	TE_USART_VSTR_RG				0x34
#define	TE_USART_BGR_RG					0x34
#define	TE_USART_RLCR_RG				0x35
#define	TE_USART_AML_RG					0x36
#define	TE_USART_AMH_RG					0x37
#define	TE_USART_GIS_RG					0x38
#define	TE_USART_IVA_RG					0x38
#define	TE_USART_IPC_RG					0x39
#define	TE_USART_ISR0_RG				0x3A
#define	TE_USART_IMR0_RG				0x3A
#define	TE_USART_ISR1_RG				0x3B
#define	TE_USART_IMR1_RG				0x3B
#define	TE_USART_PVR_RG					0x3C
#define	TE_USART_PIS_RG					0x3D
#define	TE_USART_PIM_RG					0x3D
#define	TE_USART_PCR_RG					0x3E
#define	TE_USART_CCR4_RG				0x3F

#define	TE_USART_XON_RG					0x24	// ASYNC-mode register aliases
#define	TE_USART_XOFF_RG				0x25
#define	TE_USART_TCR_RG					0x26
#define	TE_USART_DAFO_RG				0x27
#define	TE_USART_RFC_RG					0x28
#define	TE_USART_TIC_RG					0x35
#define	TE_USART_MXN_RG					0x36
#define	TE_USART_MXF_RG					0x37

#define	TE_USART_SYNL_RG				0x24	// BISYNC-mode register aliases
#define	TE_USART_SYNH_RG				0x25





//---- USS-820/825 USB controller defs
// Note!!!: no bit defs are provided for USB controller registers. 
//	        User must do it himself upon particular application.

#define TE6X_USB_DATA_BITMASK			0xff	// read-back data bitmask

	// registers offset definitions
#define	TE_USB_TXDAT_RG					0x0
#define	TE_USB_TXCNTL_RG				0x1
#define	TE_USB_TXCNTH_RG				0x2
#define	TE_USB_TXCON_RG					0x3
#define	TE_USB_TXFLG_RG					0x4
#define	TE_USB_RXDAT_RG					0x5
#define	TE_USB_RXCNTL_RG				0x6
#define	TE_USB_RXCNTH_RG				0x7
#define	TE_USB_RXCON_RG					0x8
#define	TE_USB_RXFLG_RG					0x9
#define	TE_USB_EPINDEX_RG				0xA
#define	TE_USB_EPCON_RG					0xB
#define	TE_USB_TXSTAT_RG				0xC
#define	TE_USB_RXSTAT_RG				0xD
#define	TE_USB_SOFL_RG					0xE
#define	TE_USB_SOFH_RG					0xF
#define	TE_USB_FADDR_RG					0x10
#define	TE_USB_SCR_RG					0x11
#define	TE_USB_SSR_RG					0x12
#define	TE_USB_SBI_RG					0x14
#define	TE_USB_SBI1_RG					0x15
#define	TE_USB_SBIE_RG					0x16
#define	TE_USB_SBIE1_RG					0x17
#define	TE_USB_REV_RG					0x18
#define	TE_USB_LOCK_RG					0x19
#define	TE_USB_PEND_RG					0x1A
#define	TE_USB_SCRATCH_RG				0x1B
#define	TE_USB_MCSR_RG					0x1C



#ifdef	__TE6X_SELECT_E64XX__
//---- DS1284 RTC (real-time clock) controller defs (TE64xx only)
// Notes:
//		1.	no bit defs are provided for RTC controller registers. 
//	        User must do it himself upon particular application.
//		2.	Command register (#B) of DS1284/DS1286 RTC controller must 
//			be configured with specific bits settings as IPSW=0, IBH/LO=0, 
//			PU/PL=0	in order to	match E64xx on-board h/w.

#define TE6X_RTC_DATA_BITMASK			0xff	// read-back data bitmask

	// registers offset definitions
#define	TE_RTC_CLK_SECONDS_FRACTION_RG	0x0		// x0.01sec/x0.1sec clock settings
#define	TE_RTC_CLK_SECONDS_RG			0x1		// x1sec/x10sec clock settings
#define	TE_RTC_CLK_MINUTES_RG			0x2		// x1min/x10min clock settings
#define	TE_RTC_ALARM_MINUTES_RG			0x3		// x1min/x10min alarm settings
#define	TE_RTC_CLK_HOURS_RG				0x4		// x1hour/x10hour clock settings
#define	TE_RTC_ALARM_HOURS_RG			0x5		// x1hour/x10hour alarm settings
#define	TE_RTC_CLK_DAY_OF_WEEK_RG		0x6		// day of week clock settings
#define	TE_RTC_ALARM_DAY_OF_WEEK_RG		0x7		// day of week alarm settings
#define	TE_RTC_CLK_DATE_RG				0x8		// date clock settings
#define	TE_RTC_CLK_MONTH_RG				0x9		// month clock settings
#define	TE_RTC_CLK_YEAR_RG				0xa		// year clock settings
#define	TE_RTC_CMND_RG					0xb		// command register
#define	TE_RTC_WDT_SECONDS_FRACTION_RG	0xc		// x0.01sec/x0.1sec WDT settings
#define	TE_RTC_WDT_SECONDS_RG			0xd		// x1sec/x10sec WDT settings

#define	TE_RTC_USER_RAM_SADDR			0xe		// base address for user registers area (RAM)
#define	TE_RTC_USER_RAM_EADDR			0x3f	// end address for user registers area (RAM)
#define	TE_RTC_USER_RAM_LEN				0x32	// length of user registers area (RAM)

#endif

//================================================================



//=============== Macros for I/O area control ====================
// E6x utilities rev.3x and later offer the unified utility functions synopsis:
//  - TE6X_set_..._rg(..)    - sets the control register contents
//	- TE6X_get_..._rg()      - returns contents of control register
//  - TE6X_set_...(..)       - sets bit of control register (using particular bit set value)
//  - TE6X_get_...()         - returns bit state of control register (using particular bit set value)
//  - TE6X_uset_...(ON/OFF)  - sets bit of control register using unified ON/OFF values
//  - TE6X_uget_...()        - returns bit state of control register using unified ON/OFF values
// Aliases are provided for syntax compatibility with E6x DSP utilities rev.1x/2x.



//------------- I/O data type selector for different E6x controllers ---------
// E62/E67/E6202/E6203 have all I/O peripherals aligned as LSB of 32-bit words, then
// I/O peripherals can be accessed as UNSIGNED, UNSIGNED SHORT and UNSIGNED CHAR.
//
// E64xx have all I/O peripherals conneted to EMIF-B, so these are aligned as
// LSB of 16-bit words with the corresponding address shift, so these can be
// accessed as UNSIGNED SHORT or UNSIGNED CHAR.
//
// The declared TE6X_IO_DATA_TYPE is used to unify below macros and functions and
// to make particular selection at the run-time compilation time.

#ifdef	__TE6X_SELECT_E62_E67_E620X__
typedef	unsigned				TE6X_IO_DATA_TYPE;
typedef	unsigned				TE6X_FLASH_DATA_TYPE;
#endif

#ifdef	__TE6X_SELECT_E64XX__
typedef	unsigned short			TE6X_IO_DATA_TYPE;
typedef	unsigned char			TE6X_FLASH_DATA_TYPE;
#endif
//----------------------------------------------------------------------------



//---- WDT control
	// reset WDT
#define TE6X_set_wdt_reset()			(*(TE6X_IO_DATA_TYPE *) TE6X_WDT_RESET_ADDR)=0
	// aliases for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_wdt_reset()				TE6X_set_wdt_reset()

	// WDT enable register
#define TE6X_set_wdt_enable_rg(v)		(*(TE6X_IO_DATA_TYPE *) TE6X_WDT_ENABLE_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_wdt_enable_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_WDT_ENABLE_ADDR) & TE6X_CNTR_DATA_BITMASK)

	// dedicated bit-specific macros with V = {TE6X_WDT_ENABLE_ON,TE6X_WDT_ENABLE_OFF}
#define TE6X_set_wdt_enable(v)			TE6X_set_wdt_enable_rg((TE6X_get_wdt_enable_rg() & ~TE6X_WDT_ENABLE_BITMASK) | (v & TE6X_WDT_ENABLE_BITMASK))
#define TE6X_get_wdt_enable()			(TE6X_get_wdt_enable_rg() & TE6X_WDT_ENABLE_BITMASK)
		// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_wdt_enable()				TE6X_get_wdt_enable()

	// dedicated bit-specific macros with V = {TE6X_RTC_WDT_ENABLE_ON,TE6X_RTC_WDT_ENABLE_OFF}
#define TE6X_set_rtc_wdt_enable(v)		TE6X_set_wdt_enable_rg((TE6X_get_wdt_enable_rg() & ~TE6X_RTC_WDT_ENABLE_BITMASK) | (v & TE6X_RTC_WDT_ENABLE_BITMASK))
#define TE6X_get_rtc_wdt_enable()		(TE6X_get_wdt_enable_rg() & TE6X_RTC_WDT_ENABLE_BITMASK)

		// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_rtc_wdt_enable()			TE6X_get_rtc_wdt_enable()


	// dedicated bit-specific macros with V = {TE6X_RTC_ACCESS_ENABLE_ON,TE6X_RTC_ACCESS_ENABLE_OFF}
#define TE6X_set_rtc_access_enable(v)	TE6X_set_wdt_enable_rg((TE6X_get_wdt_enable_rg() & ~TE6X_RTC_ACCESS_ENABLE_BITMASK) | (v & TE6X_RTC_ACCESS_ENABLE_BITMASK))
#define TE6X_get_rtc_access_enable()	(TE6X_get_wdt_enable_rg() & TE6X_RTC_ACCESS_ENABLE_BITMASK)

		// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_rtc_access_enable()		TE6X_get_rtc_access_enable()


	// dedicated bit-specific macros for reading xdev_id bit with V = {TE6X_XDEV_ID_ON,TE6X_XDEV_ID_OFF}
#define TE6X_get_xdev_id()				((*(TE6X_IO_DATA_TYPE *) TE6X_WDT_ENABLE_ADDR) & TE6X_XDEV_ID_BITMASK)
		// alias for style compatibility with E6x DSP utils rev.1x/2x
#define TE6X_xdev_id()					TE6X_get_xdev_id()

	// Unified macros with V = {TE6X_ON,TE6X_OFF}
#define	TE6X_uset_wdt_enable(v)			TE6X_set_wdt_enable(v)
#define	TE6X_uget_wdt_enable()			TE6X_get_wdt_enable()
#define	TE6X_uset_rtc_wdt_enable(v)		TE6X_set_rtc_wdt_enable((v<<TE6X_RTC_WDT_ENABLE_BITSHIFT))
#define	TE6X_uget_rtc_wdt_enable()		(TE6X_get_rtc_wdt_enable()>>TE6X_RTC_WDT_ENABLE_BITSHIFT)
#define	TE6X_uset_rtc_access_enable(v)	TE6X_set_rtc_access_enable((v<<TE6X_RTC_ACCESS_ENABLE_BITSHIFT))
#define	TE6X_uget_rtc_access_enable()	(TE6X_get_rtc_access_enable()>>TE6X_RTC_ACCESS_ENABLE_BITSHIFT)
#define	TE6X_uget_xdev_id()				(TE6X_get_xdev_id()>>TE6X_XDEV_ID_BITSHIFT)

	// direct bit-specific set/get macros
#define	TE6X_SET_WDT_RESET				TE6X_set_wdt_reset()

#define	TE6X_SET_WDT_ENABLE_OFF			TE6X_set_wdt_enable(TE6X_WDT_ENABLE_OFF)
#define	TE6X_SET_WDT_ENABLE_ON			TE6X_set_wdt_enable(TE6X_WDT_ENABLE_ON)
#define	TE6X_WDT_ENABLE_IS_OFF			(TE6X_get_wdt_enable()==TE6X_WDT_ENABLE_OFF)
#define	TE6X_WDT_ENABLE_IS_ON			(TE6X_get_wdt_enable()==TE6X_WDT_ENABLE_ON)

#define	TE6X_SET_RTC_WDT_ENABLE_OFF		TE6X_set_rtc_wdt_enable(TE6X_RTC_WDT_ENABLE_OFF)
#define	TE6X_SET_RTC_WDT_ENABLE_ON		TE6X_set_rtc_wdt_enable(TE6X_RTC_WDT_ENABLE_ON)
#define	TE6X_RTC_WDT_ENABLE_IS_OFF		(TE6X_get_rtc_wdt_enable()==TE6X_RTC_WDT_ENABLE_OFF)
#define	TE6X_RTC_WDT_ENABLE_IS_ON		(TE6X_get_rtc_wdt_enable()==TE6X_RTC_WDT_ENABLE_ON)

#define	TE6X_SET_RTC_ACCESS_ENABLE_OFF	TE6X_set_rtc_access_enable(TE6X_RTC_ACCESS_ENABLE_OFF)
#define	TE6X_SET_RTC_ACCESS_ENABLE_ON	TE6X_set_rtc_access_enable(TE6X_RTC_ACCESS_ENABLE_ON)
#define	TE6X_RTC_ACCESS_ENABLE_IS_OFF	(TE6X_get_rtc_access_enable()==TE6X_RTC_ACCESS_ENABLE_OFF)
#define	TE6X_RTC_ACCESS_ENABLE_IS_ON	(TE6X_get_rtc_access_enable()==TE6X_RTC_ACCESS_ENABLE_ON)

#define	TE6X_XDEV_ID_IS_OFF				(TE6X_get_xdev_id()==TE6X_XDEV_ID_OFF)
#define	TE6X_XDEV_ID_IS_ON				(TE6X_get_xdev_id()==TE6X_XDEV_ID_ON)


//---- PIOX/SIOX reset control
#define TE6X_set_pxsx_reset_rg(v)		(*(TE6X_IO_DATA_TYPE *)  TE6X_PXSX_RESET_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_pxsx_reset_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_PXSX_RESET_ADDR)  & TE6X_CNTR_DATA_BITMASK)

	// dedicated bit-specific macros with V = {TE6X_{PX/SXA/SXB}_RESET_ON,TE6X_{PX/SXA/SXB}_RESET_OFF}
#define TE6X_set_px_reset(v)			TE6X_set_pxsx_reset_rg((TE6X_get_pxsx_reset_rg() & ~TE6X_PX_RESET_BITMASK) | (v & TE6X_PX_RESET_BITMASK))
#define TE6X_get_px_reset()				(TE6X_get_pxsx_reset_rg() & TE6X_PX_RESET_BITMASK)
#define TE6X_set_sxa_reset(v)			TE6X_set_pxsx_reset_rg((TE6X_get_pxsx_reset_rg() & ~TE6X_SXA_RESET_BITMASK) | (v & TE6X_SXA_RESET_BITMASK))
#define TE6X_get_sxa_reset()			(TE6X_get_pxsx_reset_rg() & TE6X_SXA_RESET_BITMASK)
#define TE6X_set_sxb_reset(v)			TE6X_set_pxsx_reset_rg((TE6X_get_pxsx_reset_rg() & ~TE6X_SXB_RESET_BITMASK) | (v & TE6X_SXB_RESET_BITMASK))
#define TE6X_get_sxb_reset()			(TE6X_get_pxsx_reset_rg() & TE6X_SXB_RESET_BITMASK)

		// aliases for E62/E67 compatibility, which has one SIOX site only (SIOX-A)
#define TE6X_set_sx_reset(v)			TE6X_set_sxa_reset(v)
#define TE6X_get_sx_reset()				TE6X_get_sxa_reset()

	// Unified macros with V = {TE6X_PXSX_RESET_ON,TE6X_PXSX_RESET_OFF}
#define	TE6X_uset_px_reset(v)			TE6X_set_px_reset(v)
#define	TE6X_uget_px_reset()			TE6X_get_px_reset()
#define	TE6X_uset_sxa_reset(v)			TE6X_set_sxa_reset((v<<TE6X_SXA_RESET_BITSHIFT))
#define	TE6X_uget_sxa_reset()			(TE6X_get_sxa_reset()>>TE6X_SXA_RESET_BITSHIFT)
#define	TE6X_uset_sxb_reset(v)			TE6X_set_sxb_reset((v<<TE6X_SXB_RESET_BITSHIFT))
#define	TE6X_uget_sxb_reset()			(TE6X_get_sxb_reset()>>TE6X_SXB_RESET_BITSHIFT)

		// aliases for E62/E67 compatibility, which has one SIOX site only (SIOX-A)
#define TE6X_uset_sx_reset(v)			TE6X_uset_sxa_reset(v)
#define TE6X_uget_sx_reset()			TE6X_uget_sxa_reset()

	// direct bit-specific set/get macros
#define	TE6X_SET_PX_RESET_OFF			TE6X_set_px_reset(TE6X_PX_RESET_OFF)
#define	TE6X_SET_PX_RESET_ON			TE6X_set_px_reset(TE6X_PX_RESET_ON)
#define	TE6X_PX_RESET_IS_OFF			(TE6X_get_px_reset()==TE6X_PX_RESET_OFF)
#define	TE6X_PX_RESET_IS_ON				(TE6X_get_px_reset()==TE6X_PX_RESET_ON)

#define	TE6X_SET_SXA_RESET_OFF			TE6X_set_sxa_reset(TE6X_SXA_RESET_OFF)
#define	TE6X_SET_SXA_RESET_ON			TE6X_set_sxa_reset(TE6X_SXA_RESET_ON)
#define	TE6X_SXA_RESET_IS_OFF			(TE6X_get_sxa_reset()==TE6X_SXA_RESET_OFF)
#define	TE6X_SXA_RESET_IS_ON			(TE6X_get_sxa_reset()==TE6X_SXA_RESET_ON)

#define	TE6X_SET_SXB_RESET_OFF			TE6X_set_sxb_reset(TE6X_SXB_RESET_OFF)
#define	TE6X_SET_SXB_RESET_ON			TE6X_set_sxb_reset(TE6X_SXB_RESET_ON)
#define	TE6X_SXB_RESET_IS_OFF			(TE6X_get_sxb_reset()==TE6X_SXB_RESET_OFF)
#define	TE6X_SXB_RESET_IS_ON			(TE6X_get_sxb_reset()==TE6X_SXB_RESET_ON)

		// aliases for E62/E67 compatibility, which has one SIOX site only (SIOX-A)
#define	TE6X_SET_SX_RESET_OFF			TE6X_SET_SXA_RESET_OFF
#define	TE6X_SET_SX_RESET_ON			TE6X_SET_SXA_RESET_ON
#define	TE6X_SX_RESET_IS_OFF			TE6X_SXA_RESET_IS_OFF
#define	TE6X_SX_RESET_IS_ON				TE6X_SXA_RESET_IS_ON


	// aliases for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_set_pxsx_run(v)			TE6X_set_pxsx_reset_rg(v)
#define TE6X_pxsx_run()					TE6X_get_pxsx_reset_rg()
#define TE6X_set_px_run(v)				TE6X_set_px_reset(v)
#define TE6X_px_run()					TE6X_get_px_reset()
#define TE6X_set_sx_run(v)				TE6X_set_sxa_reset(v)
#define TE6X_sx_run()					TE6X_get_sxa_reset()
#define TE6X_set_sxa_run(v)				TE6X_set_sxa_reset(v)
#define TE6X_sxa_run()					TE6X_get_sxa_reset()
#define TE6X_set_sxb_run(v)				TE6X_set_sxb_reset(v)	// this macros is style-compatible with E6x DSP util rev.1x/2x
#define TE6X_sxb_run()					TE6X_get_sxb_reset()	// this macros is style-compatible with E6x DSP util rev.1x/2x


//---- device ID macros
#define TE6X_get_dev_id_rg()			((*(TE6X_IO_DATA_TYPE *) TE6X_DEV_ID_ADDR) & TE6X_CNTR_DATA_BITMASK)
#define TE6X_get_dev_id()				TE6X_get_dev_id_rg()
	// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_dev_id_rg()				TE6X_get_dev_id_rg()
#define TE6X_dev_id()					TE6X_get_dev_id_rg()

	// particular device ID board macros
#define TE62_BOARD						(TE6X_XDEV_ID_IS_OFF && ((CSR&0x02000000)==0))
#define TE67_BOARD						(TE6X_XDEV_ID_IS_OFF && ((CSR&0x02000000)!=0))
#define TE62_TE67_BOARD					(TE6X_XDEV_ID_IS_OFF)
#define TE6202_BOARD					(TE6X_XDEV_ID_IS_ON && (TE6X_dev_id_rg()==TE6202_DEV_ID))
#define TE6203_BOARD					(TE6X_XDEV_ID_IS_ON && (TE6X_dev_id_rg()==TE6203_DEV_ID))
#define TE6202_TE6203_BOARD				(TE6202_BOARD || TE6203_BOARD)
#define TE6414_BOARD					(TE6X_XDEV_ID_IS_ON && (TE6X_dev_id_rg()==TE6414_DEV_ID))
#define TE6415_BOARD					(TE6X_XDEV_ID_IS_ON && (TE6X_dev_id_rg()==TE6415_DEV_ID))
#define TE6416_BOARD					(TE6X_XDEV_ID_IS_ON && (TE6X_dev_id_rg()==TE6416_DEV_ID))
#define TE6415_TE6416_BOARD				(TE6415_BOARD || TE6416_BOARD)
#define TE64XX_BOARD					(TE6414_BOARD || TE6415_BOARD || TE6416_BOARD)


//---- board & f/w revision ID macros (E64xx only)
#ifdef	__TE6X_SELECT_E64XX__
#define TE6X_get_rev_id_rg()			((*(TE6X_IO_DATA_TYPE *) TE6X_DEV_REV_ID_ADDR) & TE6X_CNTR_DATA_BITMASK)

	// bit-specific macros
#define TE6X_get_rev_id()				(TE6X_get_rev_id_rg() & TE6X_REV_ID_BITMASK)

	// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_get_dev_rev_id()			TE6X_get_rev_id()
#define TE6X_dev_rev_id_rg()			TE6X_get_dev_rev_id_rg()
#define TE6X_dev_rev_id()				TE6X_get_dev_rev_id_rg()

	// condition check macros (for use in IF() and similar condition check operators)
#define TE64XX_REV_ID_IS_REV1A			(TE6X_get_rev_id()==TE64XX_REV_ID_REV1A)
#define TE64XX_REV_ID_IS_REV1A1			(TE6X_get_rev_id()==TE64XX_REV_ID_REV1A1)
#define TE64XX_REV_ID_IS_REV1A2			(TE6X_get_rev_id()==TE64XX_REV_ID_REV1A2)
		// aliases
#define TE64XX_REV_ID_IS_REV1			TE64XX_REV_ID_IS_REV1A
#define TE64XX_REV_ID_IS_REV1_1			TE64XX_REV_ID_IS_REV1A1
#define TE64XX_REV_ID_IS_REV1_2			TE64XX_REV_ID_IS_REV1A2

		// E6x API aliases
#define TE6X_REV_ID_IS_REV1A			TE64XX_REV_ID_IS_REV1A
#define TE6X_REV_ID_IS_REV1A1			TE64XX_REV_ID_IS_REV1A1
#define TE6X_REV_ID_IS_REV1A2			TE64XX_REV_ID_IS_REV1A2
#define TE6X_REV_ID_IS_REV1				TE6X_REV_ID_IS_REV1A
#define TE6X_REV_ID_IS_REV1_1			TE6X_REV_ID_IS_REV1A1
#define TE6X_REV_ID_IS_REV1_2			TE6X_REV_ID_IS_REV1A2


#endif


//---- DSP clock freqeuncy ID and DSP EMIF clock frequency ID macros (E64xx only)
#ifdef	__TE6X_SELECT_E64XX__

	// The DSP_CLK_ID_RG read macros must appear in the E64xx boards with f/w rev.1A-1 and later only.
	// Calling these macros in rev.1A board always return the '0' value corresponding to the 600MHz DSP clock frequency.
#define TE6X_get_dsp_clk_id_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_DSP_CLK_ID_ADDR) & TE6X_CNTR_DATA_BITMASK)

		// bit-specific macros
#define TE6X_get_dsp_clk_freq_id()		(TE6X_get_dsp_clk_id_rg() & TE6X_DSP_CLK_FREQ_ID_BITMASK)

		// alias for compatibility with E6x DSP utils rev.1x/2x
#define TE6X_dsp_clk_freq_id()			TE6X_get_dsp_clk_freq_id()


	// condition check macros (for use in IF() and similar condition check operators)
	// These macros for E64xx do not depend upon the board f/w revision ID.
	// The 600MHz DSP clock frequency appears both in the f/w rev.1A boards with DSP silicon rev.1.03
	// and boards with the f/w rev.1A-1/2.
	// The 720MHz and 1GHz DSP clock frequency appears in the boards with f/w rev.1A-1 and rev.1A-2 only.
#define TE64XX_DSP_CLK_FREQ_IS_600MHZ	((TE64XX_REV_ID_IS_REV1A && C64XX_SILREV_IS_SILREV103) || ((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_dsp_clk_freq_id()==TE64XX_DSP_CLK_FREQ_ID_600MHZ)))
#define TE64XX_DSP_CLK_FREQ_IS_720MHZ	((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_dsp_clk_freq_id()==TE64XX_DSP_CLK_FREQ_ID_720MHZ))
#define TE64XX_DSP_CLK_FREQ_IS_1000MHZ	((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_dsp_clk_freq_id()==TE64XX_DSP_CLK_FREQ_ID_1000MHZ))
		// extra macros returning pseudo ID for 500MHz TMX-grade C64xx silicon in boards with f/w rev.1A only.
		// This macros is using DSP silicon rev. ID via DSP on-chip register (defined in the C6X_DEF.H header file).
#define TE64XX_DSP_CLK_FREQ_IS_500MHZ		(TE64XX_REV_ID_IS_REV1A && C64XX_SILREV_IS_EARLIERSILREV103)

		// E6x API aliases
#define TE6X_DSP_CLK_FREQ_IS_600MHZ		TE64XX_DSP_CLK_FREQ_IS_600MHZ
#define TE6X_DSP_CLK_FREQ_IS_720MHZ		TE64XX_DSP_CLK_FREQ_IS_720MHZ
#define TE6X_DSP_CLK_FREQ_IS_1000MHZ	TE64XX_DSP_CLK_FREQ_IS_1000MHZ
#define TE6X_DSP_CLK_FREQ_IS_500MHZ		TE64XX_DSP_CLK_FREQ_IS_500MHZ


#endif


//---- universal macro, which returns DSP clock frequency (floating point value in Hz)
// This macros is different for E6x boards with C620x/C6701 DSPs and for E64xx boards with C64xx DSPs.
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define	TE6X_dsp_clk_freq()				(TE62_BOARD ? TE62_DSP_CLK_FREQ:\
											(TE67_BOARD ? TE67_DSP_CLK_FREQ:\
												(TE6202_BOARD ? TE6202_DSP_CLK_FREQ:TE6203_DSP_CLK_FREQ)))
#endif
#ifdef	__TE6X_SELECT_E64XX__
#define	TE6X_dsp_clk_freq()				(TE6X_DSP_CLK_FREQ_IS_500MHZ ? TE64XX_DSP_500MHZ_CLK_FREQ:\
											(TE6X_DSP_CLK_FREQ_IS_600MHZ ? TE64XX_DSP_600MHZ_CLK_FREQ:\
												( TE6X_DSP_CLK_FREQ_IS_720MHZ ? TE64XX_DSP_720MHZ_CLK_FREQ:TE64XX_DSP_1000MHZ_CLK_FREQ)))
#endif


//---- SBSRAM/SDRAM capacity ID macros (E64xx boards only)
//     Notes:
//		1. For rev.1A boards, the TE6X_sdram_len() macros returns the SDRAM length upon the setting of the
//		   __TE6X_SDRAM128M__ compiler key.
//		2. For rev.1A boards w/o XMEM_LEN_ID register, the TE6X_sbsram_len() macros always returns the 512Kx32 SBSRAM length.
//		3. For rev.1A-1/2 boards, the TE6X_sdram_len() and TE6X_sbsram_len() macros return the SDRAM/SBSRAM
//		   length upon the corresponding field of XMEM_LEN_ID register.
//		4. For rev.1A boards, reading the XMEM_LEN_ID register always returns the '0' values for both
//		   SDRAM_LEN_ID and SBSRAM_LEN_ID fields, that correspond to the not-installed SBSRAM/SDRAM memories.

#ifdef	__TE6X_SELECT_E64XX__

	// XMEM_LEN_ID register is available for f/w rev.1A-1 and rev.1A-2 only (!!!)
#define TE6X_get_xmem_len_id_rg()			((*(TE6X_IO_DATA_TYPE *) TE6X_XMEM_LEN_ID_ADDR) & TE6X_CNTR_DATA_BITMASK)

		// dedicated bit specific macros
#define TE6X_get_sbsram_len_id()			(TE6X_get_xmem_len_id_rg() & TE6X_SBSRAM_LEN_ID_BITMASK)
#define TE6X_get_sdram_len_id()				(TE6X_get_xmem_len_id_rg() & TE6X_SDRAM_LEN_ID_BITMASK)

		// condition check macros (for use in IF() and other conditional operators)
			//	For rev.1A boards, the SBSRAM length condition check macros return the true value
			//  for the 512Kx32 SBSRAM length only.
#define TE6X_SBSRAM_LEN_IS_0K				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sbsram_len_id()==TE6X_SBSRAM_LEN_ID_0K))
#define TE6X_SBSRAM_LEN_IS_128K				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sbsram_len_id()==TE6X_SBSRAM_LEN_ID_128K))
#define TE6X_SBSRAM_LEN_IS_256K				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sbsram_len_id()==TE6X_SBSRAM_LEN_ID_256K))
#define TE6X_SBSRAM_LEN_IS_512K				(TE64XX_REV_ID_IS_REV1A || ((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sbsram_len_id()==TE6X_SBSRAM_LEN_ID_512K)))
#define TE6X_SBSRAM_IS_NOT_INSTALLED		TE6X_SBSRAM_LEN_IS_0K
#define TE6X_SBSRAM_IS_INSTALLED			!TE6X_SBSRAM_IS_NOT_INSTALLED

			//	For rev.1A boards, the SDRAM length condition check macros return the true value for the 4M and 16M length
			//	upon the setting of the __TE6X_SDRAM128M__ compiler key.
#define TE6X_SDRAM_LEN_IS_0M				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sdram_len_id()==TE6X_SDRAM_LEN_ID_0M))

#ifndef	__TE6X_SDRAM128M__
#define TE6X_SDRAM_LEN_IS_4M				(TE64XX_REV_ID_IS_REV1A || ((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sdram_len_id()==TE6X_SDRAM_LEN_ID_4M)))
#define TE6X_SDRAM_LEN_IS_16M				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sdram_len_id()==TE6X_SDRAM_LEN_ID_16M))
#endif

#ifdef	__TE6X_SDRAM128M__
#define TE6X_SDRAM_LEN_IS_4M				((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sdram_len_id()==TE6X_SDRAM_LEN_ID_4M))
#define TE6X_SDRAM_LEN_IS_16M				(TE64XX_REV_ID_IS_REV1A || ((!TE64XX_REV_ID_IS_REV1A) && (TE6X_get_sdram_len_id()==TE6X_SDRAM_LEN_ID_16M)))
#endif


#define TE6X_SDRAM_IS_NOT_INSTALLED			TE6X_SDRAM_LEN_IS_0M
#define TE6X_SDRAM_IS_INSTALLED				!TE6X_SDRAM_IS_NOT_INSTALLED

			// aliases for length in bytes
#define TE6X_SBSRAM_LEN_IS_0KB				TE6X_SBSRAM_LEN_IS_0K
#define TE6X_SBSRAM_LEN_IS_1MB				TE6X_SBSRAM_LEN_IS_128K
#define TE6X_SBSRAM_LEN_IS_2MB				TE6X_SBSRAM_LEN_IS_256K
#define TE6X_SBSRAM_LEN_IS_4MB				TE6X_SBSRAM_LEN_IS_512K

#define TE6X_SDRAM_LEN_IS_0MB				TE6X_SDRAM_LEN_IS_0M
#define TE6X_SDRAM_LEN_IS_32MB				TE6X_SDRAM_LEN_IS_4M
#define TE6X_SDRAM_LEN_IS_128MB				TE6X_SDRAM_LEN_IS_16M


		// macros returning true SBSRAM/SDRAM memories length (in bytes)
#define TE6X_get_sbsram_len()     			(TE6X_SBSRAM_LEN_IS_0K ? TE6X_SBSRAM_LEN_0K:\
													(TE6X_SBSRAM_LEN_IS_128K ? TE6X_SBSRAM_LEN_128K:\
														(TE6X_SBSRAM_LEN_IS_256K ? TE6X_SBSRAM_LEN_256K:TE6X_SBSRAM_LEN_512K )))

#define TE6X_get_sdram_len()     			(TE6X_SDRAM_LEN_IS_0M ? TE6X_SDRAM_LEN_0M:\
													(TE6X_SDRAM_LEN_IS_4M ? TE6X_SDRAM_LEN_4M:TE6X_SDRAM_LEN_16M))
#endif


//---- SBSRAM/SDRAM length return macro for E62/E67/E6202/E6203 boards
//     These are the false macros returning maximum possible values for SBSRAM (1Mx32) and 4Mx32 for SDRAM.
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define TE6X_get_sbsram_len()     			TE6X_SBSRAM_LEN
#define TE6X_get_sdram_len()     			TE6X_SDRAM_LEN
#endif




//---- external memory control via XMEM_CNTR rg (E6202/E6203 only)
#define TE6X_set_sys_cnf_rg(v)			(*(TE6X_IO_DATA_TYPE *) TE6X_SYS_CNF_ADDR) = ((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_sys_cnf_rg()			((*(TE6X_IO_DATA_TYPE *) TE6X_SYS_CNF_ADDR) & TE6X_CNTR_DATA_BITMASK)
		// aliases for compatibility with API rev.1,2,3
#define TE6X_set_xmem_cntr_rg(v)		TE6X_set_sys_cnf_rg(v)
#define TE6X_get_xmem_cntr_rg()			TE6X_get_sys_cnf_rg()


	// dedicated bit-specific macros with V = {TE6X_XMEM_SBSRAM,TE6X_XMEM_SDRAM}
#define TE6X_set_xmem(v)				TE6X_set_sys_cnf_rg(v)
#define TE6X_get_xmem()					(TE6X_get_sys_cnf_rg() & TE6X_XMEM_BITMASK)

	// E64xx only: dedicated data read-back specific macros with V = {TE6X_MCBSP1_IF_SEL,TE6X_UTOPIA_IF_SEL}
#define TE6X_get_mcbsp1_utopia_if_sel()	(TE6X_get_sys_cnf_rg() & TE6X_MCBSP1_UTOPIA_IF_SEL_BITMASK)

	// direct bit-specific set/get macros
#define	TE6X_SET_XMEM_SBSRAM			TE6X_set_xmem(TE6X_XMEM_SBSRAM)
#define	TE6X_SET_XMEM_SDRAM				TE6X_set_xmem(TE6X_XMEM_SDRAM)
#define	TE6X_XMEM_IS_SBSRAM				(TE6X_get_xmem()==TE6X_XMEM_SBSRAM)
#define	TE6X_XMEM_IS_SDRAM				(TE6X_get_xmem()==TE6X_XMEM_SDRAM)

#define	TE6X_IF_IS_MCBSP1				(TE6X_get_mcbsp1_utopia_if_sel()==TE6X_MCBSP1_IF_SEL)
#define	TE6X_IF_IS_UTOPIA				(TE6X_get_mcbsp1_utopia_if_sel()==TE6X_UTOPIA_IF_SEL)


//---- Boot mode read-back macros (E6202/E6203/E64xx only)
#define TE6X_get_bmode_rg()				((*(TE6X_IO_DATA_TYPE *) TE6X_BMODE_ADDR) & TE6X_CNTR_DATA_BITMASK)

	// dedicated bit-specific macros with V = {TE6X_XMEM_SBSRAM,TE6X_XMEM_SDRAM}
#define TE6X_get_bmode()				TE6X_get_bmode_rg()

	// direct get macros
#define	TE6X_BMODE_IS_MAP0_NONE_SBSRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP0_NONE_SBSRAM)
#define	TE6X_BMODE_IS_MAP0_NONE_SDRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP0_NONE_SDRAM)
#define	TE6X_BMODE_IS_MAP0_FLASH8_SBSRAM	(TE6X_get_bmode()==TE6X_BMODE_MAP0_FLASH8_SBSRAM)
#define	TE6X_BMODE_IS_MAP0_FLASH8_SDRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP0_FLASH8_SDRAM)
#define	TE6X_BMODE_IS_MAP0_HPI_SBSRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP0_HPI_SBSRAM)

#define	TE6X_BMODE_IS_MAP1_NONE_SBSRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP1_NONE_SBSRAM)
#define	TE6X_BMODE_IS_MAP1_FLASH8_SBSRAM	(TE6X_get_bmode()==TE6X_BMODE_MAP1_FLASH8_SBSRAM)
#define	TE6X_BMODE_IS_MAP1_HPI_SBSRAM		(TE6X_get_bmode()==TE6X_BMODE_MAP1_HPI_SBSRAM)

#define	TE64XX_BMODE_IS_NONE				(TE6X_get_bmode()==TE64XX_BMODE_NONE)
#define	TE64XX_BMODE_IS_FLASH8				(TE6X_get_bmode()==TE64XX_BMODE_FLASH8)
#define	TE64XX_BMODE_IS_HPI					(TE6X_get_bmode()==TE64XX_BMODE_HPI)

		// logical combinations and aliases
#define	TE6X_BMODE_IS_MAP0_NONE				(TE6X_BMODE_IS_MAP0_NONE_SBSRAM	|| TE6X_BMODE_IS_MAP0_NONE_SDRAM)
#define	TE6X_BMODE_IS_MAP0_FLASH8			(TE6X_BMODE_IS_MAP0_FLASH8_SBSRAM	|| TE6X_BMODE_IS_MAP0_FLASH8_SDRAM)
#define	TE6X_BMODE_IS_MAP0_HPI				TE6X_BMODE_IS_MAP0_HPI_SBSRAM

#define	TE6X_BMODE_IS_MAP1_NONE				TE6X_BMODE_IS_MAP1_NONE_SBSRAM
#define	TE6X_BMODE_IS_MAP1_FLASH8			TE6X_BMODE_IS_MAP1_FLASH8_SBSRAM
#define	TE6X_BMODE_IS_MAP1_HPI				TE6X_BMODE_IS_MAP1_HPI_SBSRAM

#define	TE6X_BMODE_MAP_IS_MAP0				(TE6X_BMODE_IS_MAP0_NONE || TE6X_BMODE_IS_MAP0_FLASH8 || TE6X_BMODE_IS_MAP0_HPI)
#define	TE6X_BMODE_MAP_IS_MAP1				(TE6X_BMODE_IS_MAP1_NONE || TE6X_BMODE_IS_MAP1_FLASH8 || TE6X_BMODE_IS_MAP1_HPI)



//---- C6x DSP EXT_INT6/EXT_INT6 interrupt selectors setting macros
#define TE6X_set_ext_int4_sel_rg(v)		(*(TE6X_IO_DATA_TYPE *)  TE6X_EXT_INT4_SEL_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_ext_int4_sel_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_EXT_INT4_SEL_ADDR) & TE6X_CNTR_DATA_BITMASK)
#define TE6X_set_ext_int5_sel_rg(v)		(*(TE6X_IO_DATA_TYPE *)  TE6X_EXT_INT5_SEL_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_ext_int5_sel_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_EXT_INT5_SEL_ADDR) & TE6X_CNTR_DATA_BITMASK)
#define TE6X_set_ext_int6_sel_rg(v)		(*(TE6X_IO_DATA_TYPE *)  TE6X_EXT_INT6_SEL_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_ext_int6_sel_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_EXT_INT6_SEL_ADDR) & TE6X_CNTR_DATA_BITMASK)
#define TE6X_set_ext_int7_sel_rg(v)		(*(TE6X_IO_DATA_TYPE *)  TE6X_EXT_INT7_SEL_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_ext_int7_sel_rg()		((*(TE6X_IO_DATA_TYPE *) TE6X_EXT_INT7_SEL_ADDR) & TE6X_CNTR_DATA_BITMASK)
#define TE6X_set_nmi_sel_rg(v)			(*(TE6X_IO_DATA_TYPE *)  TE6X_NMI_SEL_ADDR)=((TE6X_IO_DATA_TYPE)v)
#define TE6X_get_nmi_sel_rg()			((*(TE6X_IO_DATA_TYPE *) TE6X_NMI_SEL_ADDR) & TE6X_CNTR_DATA_BITMASK)

	// dedicated bit-specific macros 
#define	TE6X_set_ext_int4_sel(v)		TE6X_set_ext_int4_sel_rg(v)
#define TE6X_get_ext_int4_sel()			TE6X_get_ext_int4_sel_rg()
#define	TE6X_set_ext_int5_sel(v)		TE6X_set_ext_int5_sel_rg(v)
#define TE6X_get_ext_int5_sel()			TE6X_get_ext_int5_sel_rg()
#define	TE6X_set_ext_int6_sel(v)		TE6X_set_ext_int6_sel_rg(v)
#define TE6X_get_ext_int6_sel()			TE6X_get_ext_int6_sel_rg()
#define	TE6X_set_ext_int7_sel(v)		TE6X_set_ext_int7_sel_rg(v)
#define TE6X_get_ext_int7_sel()			TE6X_get_ext_int7_sel_rg()
#define	TE6X_set_nmi_sel(v)				TE6X_set_nmi_sel_rg(v)
#define TE6X_get_nmi_sel()				TE6X_get_nmi_sel_rg()

	// aliases and style-compatible aliases for compatibility with E6x DSP utils rev.1x/2x 
#define TE6X_ext_int4_sel()				TE6X_get_ext_int4_sel_rg()
#define TE6X_ext_int5_sel()				TE6X_get_ext_int5_sel_rg()
#define TE6X_ext_int6_sel()				TE6X_get_ext_int6_sel_rg()
#define TE6X_ext_int7_sel()				TE6X_get_ext_int7_sel_rg()
#define TE6X_nmi_sel()					TE6X_get_nmi_sel_rg()

	// universal macros to set/get EXT_INT/EXT_NMI selector regs using EXT_INT ID
		// IDs for EXT_INT/NMI selector registers
#define	TE6X_EXT_INT4_SEL				0
#define	TE6X_EXT_INT5_SEL				1
#define	TE6X_EXT_INT6_SEL				2
#define	TE6X_EXT_INT7_SEL				3
#define	TE6X_NMI_SEL					4

		//input params: IRQ={TE6X_EXT_INTx_SEL/TE6X_NMI_SEL}, V={TE6X_EXT_INT_SEL_..}
#define	TE6X_set_ext_int_sel_rg(irq,v)	if(irq==0) TE6X_set_ext_int4_sel_rg(v); \
										  else if(irq==1) TE6X_set_ext_int5_sel_rg(v);\
										    else if(irq==2) TE6X_set_ext_int6_sel_rg(v);\
										      else if(irq==3) TE6X_set_ext_int7_sel_rg(v);\
												else TE6X_set_nmi_sel_rg(v)
#define	TE6X_get_ext_int_sel_rg(irq)	((irq==0)? TE6X_get_ext_int4_sel(): \
										  ((irq==1)? TE6X_get_ext_int5_sel():\
										    ((irq==2)? TE6X_get_ext_int6_sel():\
										      ((irq==3)? TE6X_get_ext_int7_sel():TE6X_get_nmi_sel()))))
		// macros aliases
#define	TE6X_set_ext_int_sel(irq,v)		TE6X_set_ext_int_sel_rg(irq,v)
#define	TE6X_get_ext_int_sel(irq)		TE6X_get_ext_int_sel_rg(irq)

		// direct universal set/get macros
#define	TE6X_SET_EXT_INT_SEL_OFF(irq)		TE6X_set_ext_int_sel(irq,(TE62_TE67_BOARD)? TE62_TE67_EXT_INT_SEL_OFF:TE6X_EXT_INT_SEL_OFF)
#define	TE6X_SET_EXT_INT_SEL_SXIRQ0(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_SXIRQ0)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_SXIRQ1(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_SXIRQ1)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_SXIRQ2(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_SXIRQ2)	
#define	TE6X_SET_EXT_INT_SEL_PXIRQ0(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_PXIRQ0)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_PXIRQ1(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_PXIRQ1)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_PXIRQ2(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_PXIRQ2)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_PXIRQ3(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_PXIRQ3)	// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_USART(irq)		TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_USART)
#define	TE6X_SET_EXT_INT_SEL_USB(irq)		TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_USB)
#define	TE6X_SET_EXT_INT_SEL_WDT(irq)		TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_WDT)		// E6202/E6203/E64xx only
#define	TE6X_SET_EXT_INT_SEL_RTC_WDT(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_RTC_WDT)		// E64xx only
#define	TE6X_SET_EXT_INT_SEL_RTC_ALARM(irq)	TE6X_set_ext_int_sel(irq,TE6X_EXT_INT_SEL_RTC_ALARM)	// E64xx only

#define	TE6X_EXT_INT_SEL_IS_OFF(irq)		(TE6X_get_ext_int_sel(irq)==((TE62_TE67_BOARD)? TE62_TE67_EXT_INT_SEL_OFF:TE6X_EXT_INT_SEL_OFF))
#define	TE6X_EXT_INT_SEL_IS_SXIRQ0(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_SXIRQ0)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_SXIRQ1(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_SXIRQ1)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_SXIRQ2(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_SXIRQ2)
#define	TE6X_EXT_INT_SEL_IS_PXIRQ0(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_PXIRQ0)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_PXIRQ1(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_PXIRQ1)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_PXIRQ2(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_PXIRQ2)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_PXIRQ3(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_PXIRQ3)	// E6202/E6203/E64x only
#define	TE6X_EXT_INT_SEL_IS_USART(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_USART)
#define	TE6X_EXT_INT_SEL_IS_USB(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_USB)
#define	TE6X_EXT_INT_SEL_IS_WDT(irq)		(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_WDT)		// E6202/E6203E64x only
#define	TE6X_EXT_INT_SEL_IS_RTC_WDT(irq)	(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_RTC_WDT)	// E64xx only
#define	TE6X_EXT_INT_SEL_IS_RTC_ALARM(irq)	(TE6X_get_ext_int_sel(irq)==TE6X_EXT_INT_SEL_RTC_ALARM)	// E64xx only

		// aliases for compatibility with E6x DSP utils rev.1x/2x 
#define	TE6X_SET_EXT_INT_SEL_XIRQ2(irq)		TE6X_SET_EXT_INT_SEL_SXIRQ2(irq)
#define	TE6X_EXT_INT_SEL_IS_XIRQ2(irq)		TE6X_EXT_INT_SEL_IS_SXIRQ2(irq)		


//---- macros to configure polarity of DSP EXT_INT4..7 inputs upon the E6x board type
// E62/E67: require that EXT_INT4..7 inputs are configired as active low.
// E6202/E6203/E64x: require that EXT_INT4..7 inputs are configired as active high.
// Note, that in case EXT_INT4..7 inputs are used in E62/E67 to synchronize DMA 
// transfers, then only active high synchronization must be used and polarity must be 
// positive (use macros defined in C6X_DEF.H file to set individual polarity for EXT_INT
// inputs).
#define	TE6X_INIT_EXT_INT_POLARITY		(*(TE6X_IO_DATA_TYPE *) 0x019C0008) = (TE62_TE67_BOARD) ? 0xf:0



//---- SAB82532 USART (ESCC2) access macros
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define TE6X_set_usart_rg(a,v)			(*(TE6X_IO_DATA_TYPE *) (TE6X_USART_SADDR + (((a)<<2)&0x1fc)))=((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_usart_rg(a)			((*(TE6X_IO_DATA_TYPE *) (TE6X_USART_SADDR + (((a)<<2)&0x1fc))) &0xff)
#endif	

#ifdef	__TE6X_SELECT_E64XX__
#define TE6X_set_usart_rg(a,v)			(*(TE6X_IO_DATA_TYPE *) (TE6X_USART_SADDR + (((a)<<1)&0xfe)))=((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_usart_rg(a)			((*(TE6X_IO_DATA_TYPE *) (TE6X_USART_SADDR + (((a)<<1)&0xfe))) &0xff)
#endif

	// aliases for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_usart_rg(a)				TE6X_get_usart_rg(a)			


//---- USS-820/825 USB controller registers access macros
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define TE6X_set_usb_rg(a,v)			(*(TE6X_IO_DATA_TYPE *) (TE6X_USB_SADDR + (((a)<<2)&0x7c))) =((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_usb_rg(a)				((*(TE6X_IO_DATA_TYPE *) (TE6X_USB_SADDR + (((a)<<2)&0x7c))) &0xff)
#endif	

#ifdef	__TE6X_SELECT_E64XX__
#define TE6X_set_usb_rg(a,v)			(*(TE6X_IO_DATA_TYPE *) (TE6X_USB_SADDR + (((a)<<1)&0x3e))) =((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_usb_rg(a)				((*(TE6X_IO_DATA_TYPE *) (TE6X_USB_SADDR + (((a)<<1)&0x3e))) &0xff)
#endif

	// aliases for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_usb_rg(a)					TE6X_get_usb_rg(a)		


//---- E64xx only: RTC DS1284 real-time clock controller registers access macros
#ifdef	__TE6X_SELECT_E64XX__

#define TE6X_set_rtc_rg(a,v)			(*(TE6X_IO_DATA_TYPE *) (TE6X_RTC_SADDR + (((a)<<1)&0x7e))) =((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_rtc_rg(a)				((*(TE6X_IO_DATA_TYPE *) (TE6X_RTC_SADDR + (((a)<<1)&0x7e))) &0xff)

#define TE6X_set_rtc_user_ram(a,v)		(*(TE6X_IO_DATA_TYPE *) (TE6X_RTC_SADDR + (((TE_RTC_USER_RAM_SADDR+(a%TE_RTC_USER_RAM_LEN))<<1)&0x7e))) =((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_rtc_user_ram(a)		((*(TE6X_IO_DATA_TYPE *) (TE6X_RTC_SADDR + (((TE_RTC_USER_RAM_SADDR+(a%TE_RTC_USER_RAM_LEN))<<1)&0x7e))) &0xff)

	// aliases for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_rtc_rg(a)					TE6X_get_rtc_rg(a)		
#define TE6X_rtc_user_ram(a)			TE6X_get_rtc_user_ram(a)
#endif


//---- PIOX-16 access macros
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define TE6X_set_px_rg(a,v)				(*(TE6X_IO_DATA_TYPE *) (TE6X_PIOX_SADDR + (((a)<<2)&0x3fffc)))=((TE6X_IO_DATA_TYPE)v &0xffff)
#define TE6X_get_px_rg(a)				((*(TE6X_IO_DATA_TYPE *) (TE6X_PIOX_SADDR + (((a)<<2)&0x3fffc))) &0xffff)
#endif	

#ifdef	__TE6X_SELECT_E64XX__
#define TE6X_set_px_rg(a,v)				(*(TE6X_IO_DATA_TYPE *) (TE6X_PIOX_SADDR + (((a)<<1)&0x1fffe)))=((TE6X_IO_DATA_TYPE)v &0xffff)
#define TE6X_get_px_rg(a)				((*(TE6X_IO_DATA_TYPE *) (TE6X_PIOX_SADDR + (((a)<<1)&0x1fffe))) &0xffff)
#endif

	// aliases for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_set_px(a,v)				TE6X_set_px_rg(a,v)	
#define TE6X_px(a)						TE6X_get_px_rg(a)


//---- SIOX_rev_C access macros
#ifdef	__TE6X_SELECT_E62_E67_E620X__
#define TE6X_set_sxa_revc_rg(a,v)		(*(TE6X_IO_DATA_TYPE *) (TE6X_SIOX_SADDR + (((a)<<2)&0xfc)))=((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_sxa_revc_rg(a)			((*(TE6X_IO_DATA_TYPE *) (TE6X_SIOX_SADDR + (((a)<<2)&0xfc))) &0xff)
#endif	

#ifdef	__TE6X_SELECT_E64XX__
#define TE6X_set_sxa_revc_rg(a,v)		(*(TE6X_IO_DATA_TYPE *) (TE6X_SIOX_SADDR + (((a)<<1)&0x7e)))=((TE6X_IO_DATA_TYPE)v &0xff)
#define TE6X_get_sxa_revc_rg(a)			((*(TE6X_IO_DATA_TYPE *) (TE6X_SIOX_SADDR + (((a)<<1)&0x7e))) &0xff)
#endif

	// aliases for E62/E67, which have only one SIOX site (SIOX-A)
#define TE6X_set_sx_revc_rg(a,v)	TE6X_set_sxa_revc_rg(a,v)	
#define TE6X_get_sx_revc_rg(a)		TE6X_get_sxa_revc_rg(a)

	// aliases for compatibility with E6x DSP utilities rev.1x/2x
#define TE6X_set_sx(a,v)			TE6X_set_sxa_revc_rg(a,v)	
#define TE6X_sx(a)					TE6X_get_sxa_revc_rg(a)

//======================================================================


//======== extended TE6X API functions defs/declarations (see below) ===

//--- DSP EMIF INIT
// Note!!!: this function now has parameter, which differs fm E6x utilities v.1x/2x.
//          Refer to the function body below.
int TE6X_init_dsp_emif(unsigned xmem_type);

//--- FLASH utilities
#define TE6X_FLASH_AMD_MANUF_ID			0x01
#define TE6X_FLASH_AM29F010_DEV_ID		0x20
#define TE6X_FLASH_AM29F040_DEV_ID		0xa4
#define TE6X_FLASH_SECTOR_PROTECTED		1
#define TE6X_FLASH_SECTOR_UNPROTECTED	0
#define TE6X_FLASH_DATA_BITMASK			0xff


int TE6X_flash_info(unsigned *manuf_id, unsigned *dev_id,unsigned *len,unsigned *sectors, unsigned *sector_len);
int TE6X_flash_erase_sector(unsigned a,unsigned vf);
int TE6X_flash_erase_chip(unsigned vf);
int TE6X_flash_write(unsigned a, unsigned d);
int TE6X_flash_read(unsigned a);
int TE6X_flash_sector_wrprot(unsigned a);


//--- general API functions
int TE6X_last_err();
int TE6X_err_flag();
int TE6X_clr_err_flag();


//--- returned error codes
#define	TE6X_OK							0		// no errors
#define	TE6X_PARAM_ERR					-1		// erroneous function parameter
#define	TE6X_FLASH_INV_DEV_ERR			-100	// unsupported FLASH device
#define	TE6X_FLASH_ERASE_ERR			-101	// FLASH erase error
#define	TE6X_FLASH_WR_ERR				-102	// FLASH write error
#define	TE6X_FLASH_CANNOT_WR_ERR		-103	// FLASH contents does not allow to write this data
#define	TE6X_FLASH_INV_RD_ERR			-104	// invalid FLASH read data during AUTO-SELECT

//====================================================================


/*=============== Extended TE6x DSP API functions ====================
 Extended TE6x DSP API functions include functions for DSP EMIF INIT and 
 FLASH programming control. However, in order to reduce the size of DSP code
 and in order to exclude mutiple function body appearance, any of the Extended API
 functions can be included/excluded during the compilation process. Inclusion/exclusion
 of particular Extended API function is controled via global vars, which can be defined
 in user code to perferm a particular action:

	__TE6X_EXCLUDE_INIT_DSP_EMIF__
		- if defined in user code, then the DSP EMIF INIT function are excluded
		  from the user source code. Default is INCLUDED.

	__TE6X_INCLUDE_AMD_FLASH_CONTROL__
		- if defined in user code, then the AMD FLASH PROGRAMMING CONTROL function
		  are included into the user source code. Default is EXCLUDED.
*/


#ifndef __TE6X_EXCLUDE_INIT_DSP_EMIF__
//--------------- DSP EMIF init function ------------------------------
// This function performs initialization of E6x DSP EMIF configiration registers
// in accordance with the E6x board type.
// Note!!: this function now has parameter instead of E6x DSP utils rev.1x/2x, which
//		   did not require parameter for this function.
//
// Input parameter XMEM must be defined for E6202/E6203 boards as the following and is
// ignored for E62/E67/E64xx boards:
//     == TE6X_XMEM_SBSRAM (0) for configuring CE0 as SBSRAM and SDRAM disabled
//     == TE6X_XMEM_SDRAM  (1) for configuring CE0/CE2 for SDRAM and SBSRAM disabled
//
//  This function has been updated in E6x API rev.4D to process the board f/w revision and
//  the DSP speed.

int TE6X_init_dsp_emif(unsigned xmem_type)
{

#ifdef	__TE6X_SELECT_E62_E67_E620X__
    if(TE62_BOARD)
	{
	  *C6X_EMIF_GCR=TE62_EMIF_GCR_DATA;
	  *C6X_EMIF_CE0=TE62_EMIF_CE0_DATA;
	  *C6X_EMIF_CE1=TE62_EMIF_CE1_DATA;
	  *C6X_EMIF_CE2=TE62_EMIF_CE2_DATA;
	  *C6X_EMIF_CE3=TE62_EMIF_CE3_DATA;
	  *C6X_EMIF_SDRAM=TE62_EMIF_SDRAM_DATA;
	  *C6X_EMIF_SDRAM_TIMER=TE62_EMIF_SDRAM_TIMER_DATA;
	  return(TE6X_OK);
	}

    if(TE67_BOARD)
	{
	  *C6X_EMIF_GCR=TE67_EMIF_GCR_DATA;
	  *C6X_EMIF_CE0=TE67_EMIF_CE0_DATA;
	  *C6X_EMIF_CE1=TE67_EMIF_CE1_DATA;
	  *C6X_EMIF_CE2=TE67_EMIF_CE2_DATA;
	  *C6X_EMIF_CE3=TE67_EMIF_CE3_DATA;
	  *C6X_EMIF_SDRAM=TE67_EMIF_SDRAM_DATA;
	  *C6X_EMIF_SDRAM_TIMER=TE67_EMIF_SDRAM_TIMER_DATA;
	  return(TE6X_OK);
	}

		// validate input parameter for E6202/E6203 board.
		// Return error in case input parameter is not valid.
    if(xmem_type!=TE6X_XMEM_SBSRAM && xmem_type!=TE6X_XMEM_SDRAM) return(TE6X_PARAM_ERR);

    if(TE6202_BOARD)
	{
	  *C6X_EMIF_GCR=TE6202_EMIF_GCR_DATA;
	  *C6X_EMIF_CE3=TE6202_EMIF_CE3_DATA;
	  *C6X_EMIF_CE1=TE6202_EMIF_CE1_DATA;
	  if(xmem_type==TE6X_XMEM_SBSRAM)	// check for external memory configuration
		{
		  TE6X_SET_XMEM_SBSRAM;
	  	  *C6X_EMIF_CE0=TE6202_EMIF_CE0_SBSRAM_DATA;
	  	  *C6X_EMIF_CE2=TE6202_EMIF_CE2_SBSRAM_DATA;
		}
		else
		{
		  TE6X_SET_XMEM_SDRAM;
	  	  *C6X_EMIF_CE0=TE6202_EMIF_CE0_SDRAM_DATA;
		  *C6X_EMIF_CE2=TE6202_EMIF_CE2_SDRAM_DATA;
		}

	  *C6X_EMIF_SDRAM=TE6202_EMIF_SDRAM_DATA;
	  *C6X_EMIF_SDRAM_TIMER=TE6202_EMIF_SDRAM_TIMER_DATA;
      return(TE6X_OK);
	}

	// TE6203 board is assumed
	*C6X_EMIF_GCR=TE6203_EMIF_GCR_DATA;
	*C6X_EMIF_CE3=TE6203_EMIF_CE3_DATA;
	*C6X_EMIF_CE1=TE6203_EMIF_CE1_DATA;
	if(xmem_type==TE6X_XMEM_SBSRAM)	// check for external memory configuration
	{
	  TE6X_SET_XMEM_SBSRAM;
	  *C6X_EMIF_CE0=TE6203_EMIF_CE0_SBSRAM_DATA;
	  *C6X_EMIF_CE2=TE6203_EMIF_CE2_SBSRAM_DATA;
	}
	else
	{
	  TE6X_SET_XMEM_SDRAM;
	  *C6X_EMIF_CE0=TE6203_EMIF_CE0_SDRAM_DATA;
	  *C6X_EMIF_CE2=TE6203_EMIF_CE2_SDRAM_DATA;
	}

	*C6X_EMIF_SDRAM=TE6203_EMIF_SDRAM_DATA;
	*C6X_EMIF_SDRAM_TIMER=TE6203_EMIF_SDRAM_TIMER_DATA;
    return(TE6X_OK);
#endif


#ifdef	__TE6X_SELECT_E64XX__
	// E64xx boards initialization. Input parameter is ignored.
	// EMIF-A initialization.
	// The E64x boards may differ in SDRAM capacity installed, DSP clock (EMIF clock),
	// and board f/w revision, which result in different EMIF control register settings.

		// at first, set the EMIF control registers, which settings do not depend upon the
		// board f/w revision, DSP/EMIF clock and SDRAM capacity installed.
			// set EMIF-A registers
	*C64XX_EMIFA_GCR 		= TE64XX_EMIFA_GCR_DATA;
	*C64XX_EMIFA_CE0 		= TE64XX_EMIFA_CE0_DATA;
	*C64XX_EMIFA_CE0_SEC	= TE64XX_EMIFA_CE0_SEC_DATA;
	*C64XX_EMIFA_CE1 		= TE64XX_EMIFA_CE1_DATA;
	*C64XX_EMIFA_CE2 		= TE64XX_EMIFA_CE2_DATA;
	*C64XX_EMIFA_CE2_SEC	= TE64XX_EMIFA_CE2_SEC_DATA;
	*C64XX_EMIFA_CE3 		= TE64XX_EMIFA_CE3_DATA;

			// set EMIF-B registers
	*C64XX_EMIFB_GCR 		= TE64XX_EMIFB_GCR_DATA;
	*C64XX_EMIFB_CE2 		= TE64XX_EMIFB_CE2_DATA;
	*C64XX_EMIFB_CE3 		= TE64XX_EMIFB_CE3_DATA;
				// set EMIF-B@CE0 area to the extended preset setting initially in order to
				// configure it as 16-bit ASYNC area (dafaults to 8-bit ASYNC on DSP reset) and
				// in order to be able to process the revision ID and SBSRAM/SDRAM capacity ID.
	*C64XX_EMIFB_CE0 		= TE64XX_EMIFB_CE0_DATA_PRESET;


	// SDRAM control must be initialized upon the SDRAM capacity
	// 	1. For f/w rev.1A, the SDRAM control is set upon the setting of the __TE6X_SDRAM128M__
	//	   compiler key def and the DSP clock (500MHZ TMX-grade silicon or 600MHZ silicon rev.1.03).
	// 	2. For f/w rev.1A-1 and rev.1A-2, the __TE6X_SDRAM128M__ compiler key is ignored and the
	//	   SDRAM control is set upon the SDRAM length ID and board f/w revision.

	if(TE6X_REV_ID_IS_REV1A)
	{
		if(TE6X_SDRAM_LEN_IS_16M)
		{
			if(TE6X_DSP_CLK_FREQ_IS_500MHZ)
			{
				// configure for TMX silicon with 500MHz DSP
				*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM16M_DATA_REV1A_500MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_500MHZ;
			}

			if(TE6X_DSP_CLK_FREQ_IS_600MHZ)
			{
				// configure for TMS silicon with 600MHz DSP
				*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM16M_DATA_REV1A_600MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A_600MHZ;
			}
		}

		if(TE6X_SDRAM_LEN_IS_4M)
		{
			if(TE6X_DSP_CLK_FREQ_IS_500MHZ)
			{
				// configure for TMX silicon with 500MHz DSP
				*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A_500MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_500MHZ;
			}

			if(TE6X_DSP_CLK_FREQ_IS_600MHZ)
			{
				// configure for TMS silicon with 600MHz DSP
				*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A_600MHZ;
				*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A_600MHZ;
			}
		}

		// EMIF-B initialization upon the DSP clock frequency
		if(TE6X_DSP_CLK_FREQ_IS_500MHZ)
		{
			*C64XX_EMIFA_SDRAM_EXT 	= TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_500MHZ;
			*C64XX_EMIFB_CE0 		= TE64XX_EMIFB_CE0_DATA_REV1A_500MHZ;
			*C64XX_EMIFB_CE1 		= TE64XX_EMIFB_CE1_DATA_REV1A_500MHZ;
		}

		if(TE6X_DSP_CLK_FREQ_IS_600MHZ)
		{
			*C64XX_EMIFA_SDRAM_EXT 	= TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A_600MHZ;
			*C64XX_EMIFB_CE0 		= TE64XX_EMIFB_CE0_DATA_REV1A_600MHZ;
			*C64XX_EMIFB_CE1 		= TE64XX_EMIFB_CE1_DATA_REV1A_600MHZ;
		}
	}


	if(TE6X_REV_ID_IS_REV1A1)
	{
		if(TE6X_SDRAM_LEN_IS_16M)
		{
			// configure for 16Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM16M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A1;
		}

		if(TE6X_SDRAM_LEN_IS_4M)
		{
			// configure for 4Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1;
		}

		if(TE6X_SDRAM_LEN_IS_0M)
		{
			// if SDRAM is not installed, then re-configure EMIF-A@CE2 area, which is reserved for SDRAM, as
			// the SBSRAM area (EMIF-A@CE0) in order to exclude SDRAM refresh cycles and increase the
			// system performance. The SDRAM control register and SDRAM timer register are configured as
			// that for 4Mx64 SDRAM in order to provide correct operation.
			*C64XX_EMIFA_CE2 		= TE64XX_EMIFA_CE0_DATA;
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A1;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A1;
		}

		// EMIF-B initialization upon the DSP clock frequency
		*C64XX_EMIFA_SDRAM_EXT 	= TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A1;
		*C64XX_EMIFB_CE0 		= TE64XX_EMIFB_CE0_DATA_REV1A1;
		*C64XX_EMIFB_CE1 		= TE64XX_EMIFB_CE1_DATA_REV1A1;
	}


	if(TE6X_REV_ID_IS_REV1A2)
	{
		if(TE6X_SDRAM_LEN_IS_16M)
		{
			// configure for 16Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM16M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM16M_TIMER_DATA_REV1A2;
		}

		if(TE6X_SDRAM_LEN_IS_4M)
		{
			// configure for 4Mx64 SDRAM
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2;
		}

		if(TE6X_SDRAM_LEN_IS_0M)
		{
			// if SDRAM is not installed, then re-configure EMIF-A@CE2 area, which is reserved for SDRAM, as
			// the SBSRAM area (EMIF-A@CE0) in order to exclude SDRAM refresh cycles and increase the
			// system performance. The SDRAM control register and SDRAM timer register are configured as
			// that for 4Mx64 SDRAM in order to provide correct operation.
			*C64XX_EMIFA_CE2 		= TE64XX_EMIFA_CE0_DATA;
			*C64XX_EMIFA_SDRAM 		= TE64XX_EMIFA_SDRAM4M_DATA_REV1A2;
			*C64XX_EMIFA_SDRAM_TIMER= TE64XX_EMIFA_SDRAM4M_TIMER_DATA_REV1A2;
		}

		// EMIF-B initialization upon the DSP clock frequency
		*C64XX_EMIFA_SDRAM_EXT 	= TE64XX_EMIFA_SDRAM_EXT_DATA_REV1A2;
		*C64XX_EMIFB_CE0 		= TE64XX_EMIFB_CE0_DATA_REV1A2;
		*C64XX_EMIFB_CE1 		= TE64XX_EMIFB_CE1_DATA_REV1A2;
	}

    return(TE6X_OK);
#endif
}
//----------------------------------------------------------------------
#endif





#ifdef __TE6X_INCLUDE_AMD_FLASH_CONTROL__
/*-------------- general extended API functions -------------------------
    GENERAL EXTENDED API FUNCTIONS shall be included in case of any extended
    API functions have been included.

    General extended TE6X API functions are used to handle the run-time
    errors for FLASH programming control and other extended API functions:
	TE6X_last_err
	TE6X_err_flag
	TE6X_clr_err_flag

    Each function returns error code (<0). In case the function must also
    return data (as TE6X_flash_read()), then data can be recognized in case
    returned code is >=0, and error is recognized in case returned code is
    <0.
*/

static	int _te6x_last_err =0,
		    _te6x_err_flag =0;


int TE6X_last_err()
	// TE6X_LAST_ERR function returns errpr code of last executed
	// API function.
 { return(_te6x_last_err);
 }


int TE6X_err_flag()
	// TE6X_ERR_FLAG function returns status of ERR_FLAG for API
	// functions. ERR_FLAG is set by each API function in case
	// error has been detected. ERR_FLAG can be reset by
	// TE6X_CLR_ERR_FLAG API function
 { return(_te6x_err_flag);
 }

int TE6X_clr_err_flag()
 {
   _te6x_err_flag=TE6X_OFF;
   return(TE6X_OK);
 }

	// local/non-API function,
	// used on exit fm API functions in order to set error code
static int _te6x_set_err(int e)
 {
   if(e<0)
     { _te6x_err_flag=1;
       _te6x_last_err=e;
     }
     else
       _te6x_last_err=TE6X_OK;
   return(e);
 }

//----------------------------------------------------------------------
#endif


#ifdef __TE6X_INCLUDE_AMD_FLASH_CONTROL__
/*--------------- AMD FLASH PROGRAMMING CONTROL functions --------------
   IMPORTANT NOTICE !!!
   ---------------------
    1. AMD FLASH control utilities for TE6x have been designed for AMD Am29F010B/Am29F040B
       5v-only FLASH memory chips only (PLCC-32 IC package).

       These FLASH control utilities will not run perfectly with FLASH memory chips
       from other manufacturers.

    2. FLASH control utilities (except for the TE6X_FLASH_READ function)
       require that FLASH WRITE feature is enabled via on-board J2 jumper set,
       oherwise the TE6X_FLASH_INVALID_ERR error will be returned and no
       operation will be performed. This is due FLASH control utilies
       are auto-configured via the FLASH MANUF/DEVICE info, which is
       read from the FLASH chip using AUTO-SELECT command.

       User FLASH API functions:
    		TE6X_flash_info
			TE6X_flash_erase_chip
        	TE6X_flash_erase_sector
			TE6X_flash_write
			TE6X_flash_read
			TE6X_flash_sector_prot

    3. At runtime, FLASH utility automatically detects the presence of the
       supported FLASH memory chips, and adjusts internal environment.

	4. Starting fm E6x API rev.4D, the EMIF-B@CE1 area for FLASH memory access is configured as
	   the 8-bit ASYNC area for compatibility with future E64xx board revisions. FLASH must be accessed
	   as UNSIGNED CHAR (BYTE) words. E6x API rev.4C and earlier configured the EMIF-B@CE1 area
	   as 16-bit ASYNC and allowed access to 8-bit FLASH as LSB of 16-bit datawords. FLASH data type
	   is now set via the TE6X_FLASH_DATA_TYPE data def type. No modification of application code is
	   required (except for the re-compilation) in case supplied FLASH programming utilities have been used 
	   in the user application code.
*/

#include "stddef.h"	// TI C Compiler standard header

#define _FLASH_AM29F010_LEN				0x20000
#define _FLASH_AM29F040_LEN				0x80000
#define _FLASH_AM29F010_SECTORS			8
#define _FLASH_AM29F040_SECTORS			8
#define _FLASH_AM29F_SECTOR_PROT_ID		1
#define _FLASH_AM29F_SECTOR_UNPROT_ID	0


static int	_te6x_flash_open_flag	= 0,
			_te6x_flash_manuf_id	= 0,
			_te6x_flash_dev_id	= 0,
			_te6x_flash_len		= 0,
			_te6x_flash_sectors	= 0,
			_te6x_flash_sector_len  = 0;


static int _te6x_flash_open()
	// local/non-API function, which is used to check whether
	// FLASH API has been open in order to identify FLASH chip info
	// and configure FLASH API environment
{
 	TE6X_FLASH_DATA_TYPE *fptr,rd;
 	int r;

  	// Check for FLASH API has been open already.
	// If not open, then open it.
    if (_te6x_flash_open_flag==TE6X_ON) return(TE6X_OK);	// exit if already opened

	// not open yet, now try to open FLASH utilities
    _te6x_flash_open_flag=TE6X_OFF;
    fptr = (TE6X_FLASH_DATA_TYPE *) TE6X_FLASH_SADDR;

    // execute AUTO-SELECT command
	fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0x90;
    _te6x_flash_manuf_id = fptr[0] & TE6X_FLASH_DATA_BITMASK;

    r=TE6X_FLASH_INV_DEV_ERR;	// assume error
    if(_te6x_flash_manuf_id != TE6X_FLASH_AMD_MANUF_ID) goto exit;

    // now read and verify the device ID
	//-- list of supported FLASHs can be extended !!!
    _te6x_flash_dev_id = fptr[1] & TE6X_FLASH_DATA_BITMASK;
    r=TE6X_FLASH_INV_DEV_ERR;	// assume error
    if(!(    (_te6x_flash_manuf_id == TE6X_FLASH_AMD_MANUF_ID)
          && (    _te6x_flash_dev_id != TE6X_FLASH_AM29F010_DEV_ID
		       || _te6x_flash_dev_id != TE6X_FLASH_AM29F040_DEV_ID
             )
	   	)
	  )
	   	goto exit;

	// now fill-in FLASH len , number of sectors, & sector len
    if(_te6x_flash_manuf_id == TE6X_FLASH_AMD_MANUF_ID)
	{
		switch(_te6x_flash_dev_id)
	    {
        	case TE6X_FLASH_AM29F010_DEV_ID:
				_te6x_flash_len		= _FLASH_AM29F010_LEN;
				_te6x_flash_sectors	= _FLASH_AM29F010_SECTORS;
				_te6x_flash_sector_len  = _FLASH_AM29F010_LEN/_FLASH_AM29F010_SECTORS;
				break;

            case TE6X_FLASH_AM29F040_DEV_ID:
				_te6x_flash_len		= _FLASH_AM29F040_LEN;
				_te6x_flash_sectors	= _FLASH_AM29F040_SECTORS;
				_te6x_flash_sector_len  = _FLASH_AM29F040_LEN/_FLASH_AM29F010_SECTORS;
				break;
		}
	}

	// That's it, now set FLASH utilities OPEN flag in order to exclude this procedure for 
	// all further API function calls
    _te6x_flash_open_flag=TE6X_ON;
	r=TE6X_OK;

exit:
   	// execute READ/RESET command
	// in order to enable further reads
    fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0xf0;
    rd = *fptr;			// perform false read
	r = r | (rd&0);		// this is the false operator in order to exclude warning
						// message, which appears since RD var is never used.
    return(r);			// return error code
}


int TE6X_flash_info(unsigned *manuf_id, unsigned *dev_id,
		    	    unsigned *len,unsigned *sectors, unsigned *sector_len)
	// Returns FLASH chip info (manuf code, device code, etc.) via
	// suplied params pointers.
	// Returns: error code.
{ 
	int r;

	// Check for FLASH API has been open already.
	if ((r=_te6x_flash_open())!=TE6X_OK) return(_te6x_set_err(r));

	if(manuf_id!=NULL) 	*manuf_id   =_te6x_flash_manuf_id;
	if(dev_id!=NULL) 		*dev_id     =_te6x_flash_dev_id;
	if(len!=NULL) 		*len	    =_te6x_flash_len;
	if(sectors!=NULL)		*sectors    =_te6x_flash_sectors;
	if(sector_len!=NULL)	*sector_len =_te6x_flash_sector_len;

	return(_te6x_set_err(TE6X_OK));
}


int TE6X_flash_sector_wrprot(unsigned a)
	// Returns FLASH sector write-protection status for the sector,
	// which corresponds to address A.
	// Returns: <0 error code.
	//          0  - sector is not protected
	//          1  - sector is protected
{ 
	int r;
  	TE6X_FLASH_DATA_TYPE *fptr,rd,rd1;

  	// Check for FLASH API has been open already.
	if ((r=_te6x_flash_open())!=TE6X_OK) return(_te6x_set_err(r));

	// validate params
	if (a>=_te6x_flash_len) return(_te6x_set_err(TE6X_PARAM_ERR));

    // set FLASH pointer and sector#
	fptr = (TE6X_FLASH_DATA_TYPE *)TE6X_FLASH_SADDR;

		// execute AUTO-SELECT COMMAND for sector protection status
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x90;

	// read & verify sector wr-protection status (==0 or ==1)
	r=TE6X_FLASH_INV_RD_ERR;	// assume error
	rd= fptr[((a/_te6x_flash_sector_len)*_te6x_flash_sector_len) +2] & TE6X_FLASH_DATA_BITMASK;

    if(rd!=0 && rd!=1) 
		goto exit;

	r=TE6X_OK;	// exit OK

exit:
	// execute READ/RESET command in order to enable further reads
    fptr[0x5555] = 0xaa;
    fptr[0x2aaa] = 0x55;
    fptr[0x5555] = 0xf0;
    rd1 = *fptr;			// perform false read
   	r = r | (rd1&0);		// this is the false operator in order to exclude warning
							// message, which appears since RD1 var is never used.
    return(_te6x_set_err(r<0? r:rd));
}



int TE6X_flash_erase_sector(unsigned a,unsigned vf)
	// Erase sector, which corresponds to FLASH address A (0...
	// Verify erased sector data if VF==1.
	// returns:  error code
{
	TE6X_FLASH_DATA_TYPE *fptr, rd;
  	unsigned sa, i;
  	int r;

  	// Check for FLASH API has been open already.
	if ((r=_te6x_flash_open())!=TE6X_OK) return(_te6x_set_err(r));

	// validate params
	if (   a>=_te6x_flash_len
	    || vf>1
	   )
		return(_te6x_set_err(TE6X_PARAM_ERR));

  	// set FLASH pointer and sector#
	fptr = (TE6X_FLASH_DATA_TYPE *)TE6X_FLASH_SADDR;
	sa = (a/_te6x_flash_sector_len);

	// execute SECTOR ERASE command
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x80;
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
	fptr[a]     =0x30;

    while(1)
	{
		rd = fptr[a] & TE6X_FLASH_DATA_BITMASK;

		// verify read data
		if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {
			// if timeout, then read again
	    	rd = fptr[a] & TE6X_FLASH_DATA_BITMASK;
            if(rd==0xff) break;	// exit OK
			return(_te6x_set_err(TE6X_FLASH_ERASE_ERR));	// error and exit
	    }
	}

	// perform erased sector data verification if requested
    if(vf!=0)
	{
		for(i=0,
			fptr = (TE6X_FLASH_DATA_TYPE *)(TE6X_FLASH_SADDR + (sa*_te6x_flash_sector_len));
            i<_te6x_flash_sector_len;
            i++,fptr++)
		{
			if((*fptr &TE6X_FLASH_DATA_BITMASK)!=0xff)
		    	return(_te6x_set_err(TE6X_FLASH_ERASE_ERR));
		}
	 }

	return(_te6x_set_err(TE6X_OK));	// exit OK
}



int TE6X_flash_erase_chip(unsigned vf)
	// Erase all sectors of FLASH. Verify erased data if VF==1.
	// returns:  error code
{
	TE6X_FLASH_DATA_TYPE *fptr,rd;
  	unsigned i;
  	int r;

  	// Check for FLASH API has been open already.
	if ((r=_te6x_flash_open())!=TE6X_OK) return(_te6x_set_err(r));

	// validate params
	if (vf>1) return(_te6x_set_err(TE6X_PARAM_ERR));

    // set FLASH pointer
	fptr = (TE6X_FLASH_DATA_TYPE *)TE6X_FLASH_SADDR;

	// execute ERASE CHIP command
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0x80;
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
	fptr[0x5555]=0x10;

    while(1)
	{
		rd = *fptr & TE6X_FLASH_DATA_BITMASK;

		// verify read data
		if(rd==0xff) break;		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {   	// if timeout, then read again
	    	rd = *fptr & TE6X_FLASH_DATA_BITMASK;
            if(rd==0xff) break;	// exit OK
			return(_te6x_set_err(TE6X_FLASH_ERASE_ERR));	// error and exit
	    }
	}

	// perform erased chip data verification if requested
    if(vf!=0)
	{
	   	for(i=0L;i<_te6x_flash_sector_len;i++,fptr++)
		{
			if((*fptr &TE6X_FLASH_DATA_BITMASK)!=0xff)
		    	return(_te6x_set_err(TE6X_FLASH_ERASE_ERR));
		}
	 }

	return(_te6x_set_err(TE6X_OK));	// exit OK
}


int TE6X_flash_write(unsigned a, unsigned d)
	// writes D (bits 0..7 are valid) to FLASH addr #A
	// returns:  error code
{
	TE6X_FLASH_DATA_TYPE *fptr,rd;
  	int r;

  	// Check for FLASH API has been open already.
	if ((r=_te6x_flash_open())!=TE6X_OK) return(_te6x_set_err(r));

	// validate params
	if (   a>=_te6x_flash_len
	    || d>TE6X_FLASH_DATA_BITMASK
	   )
	   	return(_te6x_set_err(TE6X_PARAM_ERR));

   	// set FLASH pointer
	fptr = (TE6X_FLASH_DATA_TYPE *)TE6X_FLASH_SADDR;

	// verify that FLASH contents at address A
	// is OK to program data D, i.e. '1' bits in D
	// shall match '1' bits in FLASH contents
	if((fptr[a] & d)!=d)
		return(_te6x_set_err(TE6X_FLASH_CANNOT_WR_ERR));

	// execute PROGRAM BYTE command
    fptr[0x5555]=0xaa;
    fptr[0x2aaa]=0x55;
    fptr[0x5555]=0xa0;
    fptr[a]=d ;

		// verify for program ends and errors
    while(1)
	{
		rd = (fptr[a]) & TE6X_FLASH_DATA_BITMASK;

		// verify read data
	  	if(rd==d) return(_te6x_set_err(TE6X_OK));		// exit OK

	  	// if not OK, then test bit D5 as timeout condition
	  	if((rd&0x20)!=0)
	    {
			// if timeout, then read again
	    	rd = fptr[a] & TE6X_FLASH_DATA_BITMASK;
            if(rd==d) return(_te6x_set_err(TE6X_OK));	// exit OK
			return(_te6x_set_err(TE6X_FLASH_WR_ERR));	// error and exit
	    }
	}
}


int TE6X_flash_read(unsigned a)
	// reads/returns data from FLASH addr #A
	// returns: <0 - error occured
	//	    >=0 - FLASH read data
	//
	// Note: TE6X_FLASH_READ function does not generally require
	//       FLASH WRITE enabled via jumper J2 in order to
	//       have FLASH utilities running perfectly.
	//       In case FLASH utilities are open, then A address is
	//       verified againg particular FLASH length, otherwise
	//       maximum FLASH/EPROM length is used.

{
	TE6X_FLASH_DATA_TYPE *fptr;

	// execute FLASH OPEN procedure in order to activate
	// FLASH utilities, however ignore returned error code
	_te6x_flash_open();

	// validate function params
	if(    (_te6x_flash_open_flag==TE6X_ON  && a>=_te6x_flash_len)
	     ||(_te6x_flash_open_flag==TE6X_OFF && a>=TE6X_FLASH_LEN)
	  )
		return(_te6x_set_err(TE6X_PARAM_ERR));

	if(_te6x_flash_open_flag==TE6X_ON)
	   	_te6x_last_err=TE6X_OK;	// no errors if FLASH OPEN is OK

   	// set FLASH pointer and read data
	fptr = (TE6X_FLASH_DATA_TYPE *)TE6X_FLASH_SADDR;
	return(fptr[a] & TE6X_FLASH_DATA_BITMASK);
}
//----------------------------------------------------------------------
#endif

//======================================================================


#endif
